@@ -17,7 +17,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include <assert.h>
 #include <string.h>
@@ -92,7 +94,7 @@ static void	gst_ffmpegdec_init		(GstFFMpegDec *ffmpegdec);
 static void	gst_ffmpegdec_dispose		(GObject      *object);
 
 static GstPadLinkReturn	gst_ffmpegdec_connect	(GstPad    *pad,
-						 GstCaps   *caps);
+						 const GstCaps  *caps);
 static void	gst_ffmpegdec_chain		(GstPad    *pad,
 						 GstData   *data);
 
@@ -141,9 +143,9 @@ gst_ffmpegdec_base_init (GstFFMpegDecClass *klass)
 
   /* pad templates */
   sinktempl = gst_pad_template_new ("sink", GST_PAD_SINK,
-				    GST_PAD_ALWAYS, params->sinkcaps, NULL);
+				    GST_PAD_ALWAYS, params->sinkcaps);
   srctempl = gst_pad_template_new ("src", GST_PAD_SRC,
-				   GST_PAD_ALWAYS, params->srccaps, NULL);
+				   GST_PAD_ALWAYS, params->srccaps);
 
   gst_element_class_add_pad_template (element_class, srctempl);
   gst_element_class_add_pad_template (element_class, sinktempl);
@@ -179,6 +181,7 @@ gst_ffmpegdec_init (GstFFMpegDec *ffmpegdec)
   gst_pad_set_link_function (ffmpegdec->sinkpad, gst_ffmpegdec_connect);
   gst_pad_set_chain_function (ffmpegdec->sinkpad, gst_ffmpegdec_chain);
   ffmpegdec->srcpad = gst_pad_new_from_template (oclass->srctempl, "src");
+  gst_pad_use_explicit_caps (ffmpegdec->srcpad);
 
   gst_element_add_pad (GST_ELEMENT (ffmpegdec), ffmpegdec->sinkpad);
   gst_element_add_pad (GST_ELEMENT (ffmpegdec), ffmpegdec->srcpad);
@@ -206,15 +209,11 @@ gst_ffmpegdec_dispose (GObject *object)
 
 static GstPadLinkReturn
 gst_ffmpegdec_connect (GstPad  *pad,
-		       GstCaps *caps)
+		       const GstCaps *caps)
 {
   GstFFMpegDec *ffmpegdec = (GstFFMpegDec *)(gst_pad_get_parent (pad));
   GstFFMpegDecClass *oclass = (GstFFMpegDecClass*)(G_OBJECT_GET_CLASS (ffmpegdec));
 
-  /* we want fixed caps */
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
-
   /* close old session */
   if (ffmpegdec->opened) {
     avcodec_close (ffmpegdec->context);
@@ -245,8 +244,8 @@ gst_ffmpegdec_connect (GstPad  *pad,
    * simply because we don't know! We only get it
    * during playback... */
   if (avcodec_open (ffmpegdec->context, oclass->in_plugin) < 0) {
-    GST_DEBUG (
-		"ffdec_%s: Failed to open FFMPEG codec",
+    avcodec_close (ffmpegdec->context);
+    GST_DEBUG ("ffdec_%s: Failed to open FFMPEG codec",
 		oclass->in_plugin->name);
     return GST_PAD_LINK_REFUSED;
   }
@@ -325,9 +324,9 @@ gst_ffmpegdec_chain (GstPad    *pad,
   gint have_data;
 
   if (!ffmpegdec->opened) {
-    gst_element_error (GST_ELEMENT (ffmpegdec),
-		       "ffdec_%s: input format was not set before data-start",
-		       oclass->in_plugin->name);
+    GST_ELEMENT_ERROR (ffmpegdec, CORE, NEGOTIATION, (NULL),
+		       ("ffdec_%s: input format was not set before data start",
+		       oclass->in_plugin->name));
     return;
   }
 
@@ -344,6 +343,16 @@ gst_ffmpegdec_chain (GstPad    *pad,
 
     switch (oclass->in_plugin->type) {
       case CODEC_TYPE_VIDEO:
+	/* workarounds, functions write to buffers:
+	 *  libavcodec/svq1.c:svq1_decode_frame writes to the given buffer.
+         *  libavcodec/svq3.c:svq3_decode_slice_header too.
+         * ffmpeg devs know about it and will fix it (they said). */
+	if (oclass->in_plugin->id == CODEC_ID_SVQ1 ||
+            oclass->in_plugin->id == CODEC_ID_SVQ3) {
+	  inbuf = gst_buffer_copy_on_write(inbuf);
+	  data = GST_BUFFER_DATA (inbuf);
+	  size = GST_BUFFER_SIZE (inbuf);
+	}
         len = avcodec_decode_video (ffmpegdec->context,
 				    ffmpegdec->picture,
 				    &have_data,
@@ -404,10 +413,10 @@ gst_ffmpegdec_chain (GstPad    *pad,
         caps = gst_ffmpeg_codectype_to_caps (oclass->in_plugin->type,
 					     ffmpegdec->context);
         if (caps == NULL ||
-            gst_pad_try_set_caps (ffmpegdec->srcpad, caps) <= 0) {
-          gst_element_error (GST_ELEMENT (ffmpegdec),
-			     "Failed to link ffmpeg decoder (%s) to next element",
-			     oclass->in_plugin->name);
+            !gst_pad_set_explicit_caps (ffmpegdec->srcpad, caps)) {
+          GST_ELEMENT_ERROR (ffmpegdec, CORE, NEGOTIATION, (NULL),
+			     ("Failed to link ffmpeg decoder (%s) to next element",
+			     oclass->in_plugin->name));
           return;
         }
       }
@@ -508,8 +517,12 @@ gst_ffmpegdec_register (GstPlugin *plugin)
     
     /* create the gtype now */
     type = g_type_register_static(GST_TYPE_ELEMENT, type_name , &typeinfo, 0);
-    if (!gst_element_register (plugin, type_name, GST_RANK_MARGINAL, type))
+    if (!gst_element_register (plugin, type_name, GST_RANK_MARGINAL, type)) {
+      g_free (type_name);
       return FALSE;
+    }
+
+    g_free (type_name);
 
     g_hash_table_insert (global_plugins, 
 		         GINT_TO_POINTER (type), 
