@@ -25,18 +25,16 @@
 
 /*#define DEBUG_ENABLED*/
 #include "gstjpegdec.h"
+#include <gst/video/video.h>
 
-extern GstPadTemplate *jpegdec_src_template, *jpegdec_sink_template;
+static GstPadTemplate *jpegdec_src_template, *jpegdec_sink_template;
 
 /* elementfactory information */
 GstElementDetails gst_jpegdec_details = {
-  "jpeg image decoder",
-  "Codec/Image",
-  "LGPL",
-  ".jpeg",
-  VERSION,
+  "JPEG image decoder",
+  "Codec/Decoder/Image",
+  "Decode images from JPEG format",
   "Wim Taymans <wim.taymans@tvd.be>",
-  "(C) 2000",
 };
 
 /* JpegDec signals and args */
@@ -50,12 +48,13 @@ enum {
   /* FILL ME */
 };
 
+static void     gst_jpegdec_base_init   (gpointer g_class);
 static void	gst_jpegdec_class_init	(GstJpegDec *klass);
 static void	gst_jpegdec_init	(GstJpegDec *jpegdec);
 
 static void	gst_jpegdec_chain	(GstPad *pad, GstData *_data);
 static GstPadLinkReturn
-		gst_jpegdec_link	(GstPad *pad, GstCaps *caps);
+		gst_jpegdec_link	(GstPad *pad, const GstCaps *caps);
 
 static GstElementClass *parent_class = NULL;
 /*static guint gst_jpegdec_signals[LAST_SIGNAL] = { 0 }; */
@@ -63,10 +62,11 @@ static GstElementClass *parent_class = NULL;
 GType
 gst_jpegdec_get_type(void) {
   static GType jpegdec_type = 0;
-
+  
   if (!jpegdec_type) {
     static const GTypeInfo jpegdec_info = {
-      sizeof(GstJpegDec),      NULL,
+      sizeof(GstJpegDec),
+      gst_jpegdec_base_init,
       NULL,
       (GClassInitFunc)gst_jpegdec_class_init,
       NULL,
@@ -80,6 +80,42 @@ gst_jpegdec_get_type(void) {
   return jpegdec_type;
 }
 
+static GstCaps*
+jpeg_caps_factory (void) 
+{
+  return gst_caps_new_simple ("image/jpeg",
+      "width",     GST_TYPE_INT_RANGE, 16, 4096,
+      "height",    GST_TYPE_INT_RANGE, 16, 4096,
+      "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE,
+      NULL);
+}
+
+static GstCaps*
+raw_caps_factory (void)
+{
+  return gst_caps_from_string (GST_VIDEO_CAPS_YUV ("I420"));
+}
+
+static void
+gst_jpegdec_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstCaps *raw_caps, *jpeg_caps;
+  
+  raw_caps = raw_caps_factory ();
+  jpeg_caps = jpeg_caps_factory ();
+  
+  jpegdec_sink_template = gst_pad_template_new ("sink", GST_PAD_SINK, 
+						GST_PAD_ALWAYS, 
+						jpeg_caps);
+  jpegdec_src_template = gst_pad_template_new ("src", GST_PAD_SRC, 
+					       GST_PAD_ALWAYS, 
+					       raw_caps);
+  gst_element_class_add_pad_template (element_class, jpegdec_sink_template);
+  gst_element_class_add_pad_template (element_class, jpegdec_src_template);
+  gst_element_class_set_details (element_class, &gst_jpegdec_details);
+}
+
 static void
 gst_jpegdec_class_init (GstJpegDec *klass)
 {
@@ -131,6 +167,7 @@ gst_jpegdec_init (GstJpegDec *jpegdec)
   gst_pad_set_chain_function(jpegdec->sinkpad,gst_jpegdec_chain);
   gst_pad_set_link_function(jpegdec->sinkpad, gst_jpegdec_link);
   jpegdec->srcpad = gst_pad_new_from_template (jpegdec_src_template, "src");
+  gst_pad_use_explicit_caps (jpegdec->srcpad);
   gst_element_add_pad(GST_ELEMENT(jpegdec),jpegdec->srcpad);
 
   /* initialize the jpegdec decoder state */
@@ -161,28 +198,30 @@ gst_jpegdec_init (GstJpegDec *jpegdec)
 }
 
 static GstPadLinkReturn
-gst_jpegdec_link (GstPad *pad, GstCaps *caps)
+gst_jpegdec_link (GstPad *pad, const GstCaps *caps)
 {
   GstJpegDec *jpegdec = GST_JPEGDEC (gst_pad_get_parent (pad));
+  GstStructure *structure;
+  GstCaps *srccaps;
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
+  structure = gst_caps_get_structure (caps, 0);
 
-  gst_caps_get (caps,
-		"framerate", &jpegdec->fps,
-		"width",     &jpegdec->width,
-		"height",    &jpegdec->height,
-		NULL);
+  gst_structure_get_double (structure, "framerate", &jpegdec->fps);
+  gst_structure_get_int (structure, "width",     &jpegdec->width);
+  gst_structure_get_int (structure, "height",    &jpegdec->height);
 
-  caps = GST_CAPS_NEW ("jpegdec_srccaps",
-		       "video/x-raw-yuv",
-			 "format",    GST_PROPS_FOURCC (
-			   GST_MAKE_FOURCC ('I','4','2','0')),
-			 "width",     GST_PROPS_INT (jpegdec->width),
-			 "height",    GST_PROPS_INT (jpegdec->height),
-			 "framerate", GST_PROPS_FLOAT (jpegdec->fps));
+  srccaps = gst_caps_new_simple ("video/x-raw-yuv",
+      "format",    GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I','4','2','0'),
+      "width",     G_TYPE_INT, jpegdec->width,
+      "height",    G_TYPE_INT, jpegdec->height,
+      "framerate", G_TYPE_DOUBLE, jpegdec->fps,
+      NULL);
 
-  return gst_pad_try_set_caps (jpegdec->srcpad, caps);
+  /* at this point, we're pretty sure that this will be the output
+   * format, so we'll set it. */
+  gst_pad_set_explicit_caps (jpegdec->srcpad, srccaps);
+
+  return GST_PAD_LINK_OK;
 }
 
 /* shamelessly ripped from jpegutils.c in mjpegtools */
@@ -371,16 +410,13 @@ gst_jpegdec_chain (GstPad *pad, GstData *_data)
     jpegdec->line[2] = g_realloc(jpegdec->line[2], height*sizeof(char*));
     jpegdec->height = height;
 
-    gst_pad_try_set_caps (jpegdec->srcpad, 
-		          GST_CAPS_NEW (
-			    "jpegdec_caps",
-			    "video/x-raw-yuv",
-			      "format",    GST_PROPS_FOURCC (
-				GST_MAKE_FOURCC ('I','4','2','0')),
-			      "width",     GST_PROPS_INT (width),
-			      "height",    GST_PROPS_INT (height),
-			      "framerate", GST_PROPS_FLOAT (jpegdec->fps) 
-			  ));
+    gst_pad_set_explicit_caps (jpegdec->srcpad,
+        gst_caps_new_simple ("video/x-raw-yuv",
+          "format",    GST_TYPE_FOURCC, GST_MAKE_FOURCC ('I','4','2','0'),
+          "width",     G_TYPE_INT, width,
+          "height",    G_TYPE_INT, height,
+          "framerate", G_TYPE_DOUBLE, jpegdec->fps,
+          NULL));
   }
 
   /* mind the swap, jpeglib outputs blue chroma first */
