@@ -22,18 +22,16 @@
 #include <string.h>
 #include <gst/gst.h>
 #include "gstpngenc.h"
+#include <gst/video/video.h>
 
 #define MAX_HEIGHT		4096
 
 
 GstElementDetails gst_pngenc_details = {
   "PNG encoder",
-  "Codec/Image",
-  "LGPL",
+  "Codec/Encoder/Image",
   "Encode a video frame to a .png image",
-  VERSION,
   "Jeremy SIMON <jsimon13@yahoo.fr>",
-  "(C) 2000 Donald Graft",
 };
 
 
@@ -49,11 +47,14 @@ enum
   ARG_0
 };
 
+static void     gst_pngenc_base_init    (gpointer g_class);
 static void	gst_pngenc_class_init	(GstPngEncClass *klass);
 static void	gst_pngenc_init		(GstPngEnc *pngenc);
 
 static void	gst_pngenc_chain	(GstPad *pad, GstData *_data);
 
+GstPadTemplate *pngenc_src_template, *pngenc_sink_template;
+
 static GstElementClass *parent_class = NULL;
 
 
@@ -74,7 +75,8 @@ GType gst_pngenc_get_type (void)
 
   if (!pngenc_type) {
     static const GTypeInfo pngenc_info = {
-      sizeof (GstPngEncClass), NULL,
+      sizeof (GstPngEncClass),
+      gst_pngenc_base_init,
       NULL,
       (GClassInitFunc) gst_pngenc_class_init,
       NULL,
@@ -90,6 +92,45 @@ GType gst_pngenc_get_type (void)
   return pngenc_type;
 }
 
+static GstCaps*
+png_caps_factory (void)
+{
+  return gst_caps_new_simple ("video/x-png",
+      "width",     GST_TYPE_INT_RANGE, 16, 4096,
+      "height",    GST_TYPE_INT_RANGE, 16, 4096,
+      "framerate", GST_TYPE_DOUBLE_RANGE, 0.0, G_MAXDOUBLE,
+      NULL);
+}
+
+
+static GstCaps*
+raw_caps_factory (void)
+{ 
+  return gst_caps_from_string (GST_VIDEO_CAPS_RGB);
+}
+
+static void
+gst_pngenc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstCaps *raw_caps, *png_caps;
+  
+  raw_caps = raw_caps_factory ();
+  png_caps = png_caps_factory ();
+
+  pngenc_sink_template = gst_pad_template_new ("sink", GST_PAD_SINK,
+					       GST_PAD_ALWAYS,
+					       raw_caps);
+  
+  pngenc_src_template = gst_pad_template_new ("src", GST_PAD_SRC,
+					      GST_PAD_ALWAYS,
+					      png_caps);
+  
+  gst_element_class_add_pad_template (element_class, pngenc_sink_template);
+  gst_element_class_add_pad_template (element_class, pngenc_src_template);
+  gst_element_class_set_details (element_class, &gst_pngenc_details);
+}
+
 static void
 gst_pngenc_class_init (GstPngEncClass *klass)
 {
@@ -104,26 +145,24 @@ gst_pngenc_class_init (GstPngEncClass *klass)
 
 
 static GstPadLinkReturn
-gst_pngenc_sinklink (GstPad *pad, GstCaps *caps)
+gst_pngenc_sinklink (GstPad *pad, const GstCaps *caps)
 {
   GstPngEnc *pngenc;
-  gfloat fps;
+  gdouble fps;
+  GstStructure *structure;
 
   pngenc = GST_PNGENC (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_int (structure, "width", &pngenc->width);
+  gst_structure_get_int (structure, "height", &pngenc->height);
+  gst_structure_get_double (structure, "framerate", &fps);
+  gst_structure_get_int (structure, "bpp", &pngenc->bpp);
 
-  gst_caps_get_int (caps, "width", &pngenc->width);
-  gst_caps_get_int (caps, "height", &pngenc->height);
-  gst_caps_get_float (caps, "framerate", &fps);
-  gst_caps_get_int (caps, "bpp", &pngenc->bpp);
-
-  caps = GST_CAPS_NEW ("png_src",
-		       "video/x-png",
-			 "framerate", GST_PROPS_FLOAT (fps),
-			 "width",     GST_PROPS_INT (pngenc->width),
-			 "height",    GST_PROPS_INT (pngenc->height));
+  caps = gst_caps_new_simple ("video/x-png",
+      "framerate", G_TYPE_DOUBLE, fps,
+      "width",     G_TYPE_INT, pngenc->width,
+      "height",    G_TYPE_INT, pngenc->height, NULL);
 
   return gst_pad_try_set_caps (pngenc->srcpad, caps);
 }
