@@ -20,13 +20,9 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
-#include <gst/gst.h>
-
-GST_DEBUG_CATEGORY_EXTERN (GST_CAT_SEEK);
 
 #include <string.h>
-#include <mad.h>
-#include "id3tag.h"
+#include "gstmad.h"
 
 #define GST_TYPE_MAD \
   (gst_mad_get_type())
@@ -73,8 +69,7 @@ struct _GstMad {
   gboolean	half;
   gboolean	ignore_crc;
 
-  GstCaps	*metadata;
-  GstCaps	*streaminfo;
+  GstTagList *	tags;
 
   /* negotiated format */
   gint		rate;
@@ -112,34 +107,31 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (mad_src_template_factory,
+static GstStaticPadTemplate mad_src_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "mad_src",
-    "audio/x-raw-int",
-      "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
-      "signed",      GST_PROPS_BOOLEAN (TRUE),
-      "width",       GST_PROPS_INT (16),
-      "depth",       GST_PROPS_INT (16),
-      "rate",        GST_PROPS_INT_RANGE (11025, 48000),
-      "channels",    GST_PROPS_INT_RANGE (1, 2)
+  GST_STATIC_CAPS ("audio/x-raw-int, "
+      "endianness = (int) " G_STRINGIFY (G_BYTE_ORDER) ", "
+      "signed = (boolean) true, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = (int) [ 11025, 48000 ], "
+      "channels = (int) [ 1, 2 ]"
   )
-)
+);
 
-GST_PAD_TEMPLATE_FACTORY (mad_sink_template_factory,
+static GstStaticPadTemplate mad_sink_template_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "mad_sink",
-    "audio/mpeg",
-      /* we don't need channel/rate ... */
-      "mpegversion", GST_PROPS_INT (1),
-      "layer", GST_PROPS_INT_RANGE (1, 3)
+  GST_STATIC_CAPS ("audio/mpeg, "
+      "mpegversion = (int) 1, "
+      "layer = (int) [ 1, 3 ]"
   )
-)
+);
 
 static void		gst_mad_base_init	(gpointer g_class);
 static void		gst_mad_class_init	(GstMadClass *klass);
@@ -246,6 +238,7 @@ gst_mad_emphasis_get_type(void) {
     {MAD_EMPHASIS_NONE,       "0", "None"},
     {MAD_EMPHASIS_50_15_US,   "1", "50/15 Microseconds"},
     {MAD_EMPHASIS_CCITT_J_17, "2", "CCITT J.17"},
+    {MAD_EMPHASIS_RESERVED,   "3", "Reserved"},
     { 0, NULL, NULL},
   };
   if (!mad_emphasis_type) {
@@ -260,9 +253,9 @@ gst_mad_base_init (gpointer g_class)
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
   
   gst_element_class_add_pad_template (element_class,
-		  GST_PAD_TEMPLATE_GET (mad_sink_template_factory));
+		  gst_static_pad_template_get (&mad_sink_template_factory));
   gst_element_class_add_pad_template (element_class,
-		  GST_PAD_TEMPLATE_GET (mad_src_template_factory));
+		  gst_static_pad_template_get (&mad_src_template_factory));
   gst_element_class_set_details (element_class, &gst_mad_details);
 }
 static void
@@ -294,12 +287,6 @@ gst_mad_class_init (GstMadClass *klass)
   g_object_class_install_property (gobject_class, ARG_IGNORE_CRC,
     g_param_spec_boolean ("ignore_crc", "Ignore CRC", "Ignore CRC errors",
 		          FALSE, G_PARAM_READWRITE));
-  g_object_class_install_property (gobject_class, ARG_METADATA,
-    g_param_spec_boxed ("metadata", "Metadata", "Metadata",
-                        GST_TYPE_CAPS, G_PARAM_READABLE));
-  g_object_class_install_property (gobject_class, ARG_STREAMINFO,
-    g_param_spec_boxed ("streaminfo", "Streaminfo", "Streaminfo",
-                        GST_TYPE_CAPS, G_PARAM_READABLE));
 }
 
 static void
@@ -307,14 +294,14 @@ gst_mad_init (GstMad *mad)
 {
   /* create the sink and src pads */
   mad->sinkpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (mad_sink_template_factory), "sink");
+		  gst_static_pad_template_get (&mad_sink_template_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (mad), mad->sinkpad);
   gst_pad_set_chain_function (mad->sinkpad, GST_DEBUG_FUNCPTR (gst_mad_chain));
   gst_pad_set_convert_function (mad->sinkpad, GST_DEBUG_FUNCPTR (gst_mad_convert_sink));
   gst_pad_set_formats_function (mad->sinkpad, GST_DEBUG_FUNCPTR (gst_mad_get_formats));
 
   mad->srcpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (mad_src_template_factory), "src");
+		  gst_static_pad_template_get (&mad_src_template_factory), "src");
   gst_element_add_pad (GST_ELEMENT (mad), mad->srcpad);
   gst_pad_set_event_function (mad->srcpad, GST_DEBUG_FUNCPTR (gst_mad_src_event));
   gst_pad_set_event_mask_function (mad->srcpad, GST_DEBUG_FUNCPTR (gst_mad_get_event_masks));
@@ -322,6 +309,7 @@ gst_mad_init (GstMad *mad)
   gst_pad_set_query_type_function (mad->srcpad, GST_DEBUG_FUNCPTR (gst_mad_get_query_types));
   gst_pad_set_convert_function (mad->srcpad, GST_DEBUG_FUNCPTR (gst_mad_convert_src));
   gst_pad_set_formats_function (mad->srcpad, GST_DEBUG_FUNCPTR (gst_mad_get_formats));
+  gst_pad_use_explicit_caps (mad->srcpad);
 
   mad->tempbuffer = g_malloc (MAD_BUFFER_MDLEN * 3);
   mad->tempsize = 0;
@@ -337,8 +325,7 @@ gst_mad_init (GstMad *mad)
   mad->segment_start = 0;
   mad->header.mode = -1;
   mad->header.emphasis = -1;
-  mad->metadata = NULL;
-  mad->streaminfo = NULL;
+  mad->tags = NULL;
 
   mad->half = FALSE;
   mad->ignore_crc = FALSE;
@@ -625,7 +612,7 @@ index_seek (GstMad *mad, GstPad *pad, GstEvent *event)
     if (gst_index_entry_assoc_map (entry, *try_formats, &value)) {
       /* lookup succeeded, create the seek */
 
-      GST_CAT_DEBUG (GST_CAT_SEEK, "index %s %" G_GINT64_FORMAT
+      GST_DEBUG ("index %s %" G_GINT64_FORMAT
 		     " -> %s %" G_GINT64_FORMAT,
 		     gst_format_get_details (GST_EVENT_SEEK_FORMAT (event))->nick,
 		     GST_EVENT_SEEK_OFFSET (event),
@@ -799,54 +786,12 @@ gst_mad_get_property (GObject *object, guint prop_id,
     case ARG_IGNORE_CRC:
       g_value_set_boolean (value, mad->ignore_crc);
       break;
-    case ARG_METADATA:
-      g_value_set_boxed (value, mad->metadata);
-      break;
-    case ARG_STREAMINFO:
-      g_value_set_boxed (value, mad->streaminfo);
-      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
 
-static GstCaps*
-gst_mad_get_streaminfo (GstMad *mad)
-{
-  GstCaps *caps;
-  GstProps *props;
-  GstPropsEntry *entry;
-  GEnumValue *value;
-  GEnumClass *klass;
-
-  props = gst_props_empty_new ();
-
-  entry = gst_props_entry_new ("layer", GST_PROPS_INT (mad->header.layer));
-  gst_props_add_entry (props, (GstPropsEntry *) entry);
-
-  klass = g_type_class_ref (GST_TYPE_MAD_MODE);
-  value = g_enum_get_value (klass,
-		            mad->header.mode);
-  if (value)
-    entry = gst_props_entry_new ("mode", GST_PROPS_STRING (value->value_nick));
-  g_type_class_unref (klass);
-  gst_props_add_entry (props, (GstPropsEntry *) entry);
-
-  klass = g_type_class_ref (GST_TYPE_MAD_EMPHASIS);
-  value = g_enum_get_value (klass,
-		            mad->header.emphasis);
-  if (value)
-    entry = gst_props_entry_new ("emphasis", GST_PROPS_STRING (value->value_nick));
-  g_type_class_unref (klass);
-  gst_props_add_entry (props, (GstPropsEntry *) entry);
-
-  caps = gst_caps_new ("mad_streaminfo",
-		       "application/x-gst-streaminfo",
-		       props);
-  return caps;
-}
-
 static void
 gst_mad_update_info (GstMad *mad)
 {
@@ -882,148 +827,10 @@ G_STMT_START{							\
   }
   mad->new_header = FALSE;
 
-  if (changed) {
-    gst_caps_replace_sink (&mad->streaminfo, gst_mad_get_streaminfo (mad));
-    g_object_notify (G_OBJECT (mad), "streaminfo");
-  }
 #undef CHECK_HEADER
 
 }
 
-/* gracefuly ripped from madplay */
-static GstCaps*
-id3_to_caps(struct id3_tag const *tag)
-{
-  unsigned int i;
-  struct id3_frame const *frame;
-  id3_ucs4_t const *ucs4;
-  id3_utf8_t *utf8;
-  GstProps *props;
-  GstPropsEntry *entry;
-  GstCaps *caps;
-  GList *values;
-
-  struct {
-    char const *id;
-    char const *name;
-  } const info[] = {
-    { ID3_FRAME_TITLE,   "Title"        },
-    { "TIT3",            "Subtitle"     },
-    { "TCOP",            "Copyright"    },
-    { "TPRO",            "Produced"     },
-    { "TCOM",            "Composer"     },
-    { ID3_FRAME_ARTIST,  "Artist"       },
-    { "TPE2",            "Orchestra"    },
-    { "TPE3",            "Conductor"    },
-    { "TEXT",            "Lyricist"     },
-    { ID3_FRAME_ALBUM,   "Album"        },
-    { ID3_FRAME_YEAR,    "Year"         },
-    { ID3_FRAME_TRACK,   "Track"        },
-    { "TPUB",            "Publisher"    },
-    { ID3_FRAME_GENRE,   "Genre"        },
-    { "TRSN",            "Station"      },
-    { "TENC",            "Encoder"      },
-  };
-
-  /* text information */
-  props = gst_props_empty_new ();
-
-  for (i = 0; i < sizeof(info) / sizeof(info[0]); ++i) {
-    union id3_field const *field;
-    unsigned int nstrings, namelen, j;
-    char const *name;
-
-    frame = id3_tag_findframe(tag, info[i].id, 0);
-    if (frame == 0)
-      continue;
-
-    field    = &frame->fields[1];
-    nstrings = id3_field_getnstrings(field);
-
-    name = info[i].name;
-
-    if (name) {
-      namelen = name ? strlen(name) : 0;
-
-      values = NULL;
-      for (j = 0; j < nstrings; ++j) {
-        ucs4 = id3_field_getstrings(field, j);
-        g_assert(ucs4);
-
-        if (strcmp(info[i].id, ID3_FRAME_GENRE) == 0)
-	  ucs4 = id3_genre_name(ucs4);
-
-        utf8 = id3_ucs4_utf8duplicate(ucs4);
-        if (utf8 == 0)
-	  goto fail;
-
-        entry = gst_props_entry_new (name, GST_PROPS_STRING_TYPE, utf8);
-	values = g_list_prepend (values, entry);
-        free(utf8);
-      }
-      if (values) {
-        values = g_list_reverse (values);
-
-        if (g_list_length (values) == 1) {
-          gst_props_add_entry (props, (GstPropsEntry *) values->data);
-        }
-        else {
-          entry = gst_props_entry_new(name, GST_PROPS_GLIST_TYPE, values);
-          gst_props_add_entry (props, (GstPropsEntry *) entry);
-        }
-        g_list_free (values);
-      }
-    }
-  }
-
-  values = NULL;
-  i = 0;
-  while ((frame = id3_tag_findframe(tag, ID3_FRAME_COMMENT, i++))) {
-    ucs4 = id3_field_getstring(&frame->fields[2]);
-    g_assert(ucs4);
-
-    if (*ucs4)
-      continue;
-
-    ucs4 = id3_field_getfullstring(&frame->fields[3]);
-    g_assert(ucs4);
-
-    utf8 = id3_ucs4_utf8duplicate(ucs4);
-    if (utf8 == 0)
-      goto fail;
-
-    entry = gst_props_entry_new ("Comment", GST_PROPS_STRING_TYPE, utf8);
-    values = g_list_prepend (values, entry);
-    free(utf8);
-  }
-  if (values) {
-    values = g_list_reverse (values);
-
-    if (g_list_length (values) == 1) {
-      gst_props_add_entry (props, (GstPropsEntry *) values->data);
-    }
-    else {
-      entry = gst_props_entry_new("Comment", GST_PROPS_GLIST_TYPE, values);
-      gst_props_add_entry (props, (GstPropsEntry *) entry);
-    }
-    g_list_free (values);
-  }
-
-  gst_props_debug (props);
-
-  caps = gst_caps_new ("mad_metadata",
-		       "application/x-gst-metadata",
-		       props);
-  if (0) {
-fail:
-    g_warning ("mad: could not parse ID3 tag");
-
-    return NULL;
-  }
-
-  return caps;
-}
-
 static void
 gst_mad_handle_event (GstPad *pad, GstBuffer *buffer)
 {
@@ -1185,7 +992,7 @@ gst_mad_chain (GstPad *pad, GstData *_data)
 	  break;
 	}
         if (!MAD_RECOVERABLE (mad->stream.error)) {
-          gst_element_error (GST_ELEMENT (mad), "fatal error decoding stream");
+          GST_ELEMENT_ERROR (mad, STREAM, DECODE, (NULL), (NULL));
           return;
         }
 	else if (mad->stream.error == MAD_ERROR_LOSTSYNC) {
@@ -1212,9 +1019,22 @@ gst_mad_chain (GstPad *pad, GstData *_data)
 
 	    tag = id3_tag_parse (data, tagsize);
 	    if (tag) {
-              gst_caps_replace_sink (&mad->metadata, id3_to_caps (tag));
+	      GstTagList *list;
+
+	      list = gst_mad_id3_to_tag_list (tag);
 	      id3_tag_delete (tag);
-	      g_object_notify (G_OBJECT (mad), "metadata");
+	      GST_DEBUG_OBJECT (mad, "found tag");
+	      gst_element_found_tags (GST_ELEMENT (mad), list);
+	      if (mad->tags) {
+		gst_tag_list_insert (mad->tags, list, GST_TAG_MERGE_PREPEND);
+	      } else {
+		mad->tags = gst_tag_list_copy (list);
+	      }
+	      if (GST_PAD_IS_USABLE (mad->srcpad)) {
+		gst_pad_push (mad->srcpad, GST_DATA (gst_event_new_tag (list)));
+	      } else {
+		gst_tag_list_free (list);
+	      }
 	    }
 	  }
 	}
@@ -1243,24 +1063,15 @@ gst_mad_chain (GstPad *pad, GstData *_data)
 
 	/* we set the caps even when the pad is not connected so they
 	 * can be gotten for streaminfo */
-        if (gst_pad_try_set_caps (mad->srcpad,
-	    gst_caps_new (
-	      "mad_src",
-              "audio/x-raw-int",
-              gst_props_new (
-                "endianness",  GST_PROPS_INT (G_BYTE_ORDER),
-                "signed",      GST_PROPS_BOOLEAN (TRUE),
-                "width",       GST_PROPS_INT (16),
-                "depth",       GST_PROPS_INT (16),
-                "rate",        GST_PROPS_INT (rate),
-                "channels",    GST_PROPS_INT (nchannels),
-                NULL))) <= 0)
-	{
-	  if (!gst_pad_recover_caps_error (mad->srcpad, NULL)) {
-	    gst_buffer_unref (buffer);
-	    return;
-	  }
-        }
+        gst_pad_set_explicit_caps (mad->srcpad,
+	    gst_caps_new_simple ("audio/x-raw-int",
+	      "endianness",  G_TYPE_INT, G_BYTE_ORDER,
+	      "signed",      G_TYPE_BOOLEAN, TRUE,
+	      "width",       G_TYPE_INT, 16,
+	      "depth",       G_TYPE_INT, 16,
+	      "rate",        G_TYPE_INT, rate,
+	      "channels",    G_TYPE_INT, nchannels,
+	      NULL));
 	mad->channels = nchannels;
 	mad->rate = rate;
       }
@@ -1405,8 +1216,10 @@ gst_mad_change_state (GstElement *element)
       mad_frame_finish (&mad->frame);
       mad_stream_finish (&mad->stream);
       mad->restart = TRUE;
-      gst_caps_replace (&mad->metadata, NULL);
-      gst_caps_replace (&mad->streaminfo, NULL);
+      if (mad->tags) {
+	gst_tag_list_free (mad->tags);
+	mad->tags = NULL;
+      }
       break;
     case GST_STATE_READY_TO_NULL:
       break;
@@ -1415,25 +1228,3 @@ gst_mad_change_state (GstElement *element)
   return parent_class->change_state (element);
 }
 
-static gboolean
-plugin_init (GstPlugin *plugin)
-{
-  /* create an elementfactory for the mad element */
-  if (!gst_element_register (plugin, "mad", GST_RANK_PRIMARY, GST_TYPE_MAD))
-    return FALSE;
-
-  return TRUE;
-}
-
-GST_PLUGIN_DEFINE (
-  GST_VERSION_MAJOR,
-  GST_VERSION_MINOR,
-  "mad",
-  "mp3 decoding based on the mad library",
-  plugin_init,
-  VERSION,
-  "GPL",
-  GST_COPYRIGHT,
-  GST_PACKAGE,
-  GST_ORIGIN
-)
