@@ -18,25 +18,26 @@
  */
 
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
 #include <stdlib.h>
 #include <string.h>
 #include <time.h>
 #include <vorbis/vorbisenc.h>
 
+#include <gst/gsttaginterface.h>
 #include "vorbisenc.h"
 
-extern GstPadTemplate *gst_vorbisenc_src_template, *gst_vorbisenc_sink_template;
+static GstPadTemplate *gst_vorbisenc_src_template, *gst_vorbisenc_sink_template;
 
 /* elementfactory information */
 GstElementDetails vorbisenc_details = {
   "Ogg Vorbis encoder",
-  "Codec/Audio/Encoder",
-  "LGPL",
+  "Codec/Encoder/Audio",
   "Encodes audio in OGG Vorbis format",
-  VERSION,
-  "Monty <monty@xiph.org>, " 
+  "Monty <monty@xiph.org>, "
   "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2000",
 };
 
 /* VorbisEnc signals and args */
@@ -54,7 +55,6 @@ enum
   ARG_MIN_BITRATE,
   ARG_QUALITY,
   ARG_SERIAL,
-  ARG_METADATA,
   ARG_MANAGED,
   ARG_LAST_MESSAGE,
 };
@@ -69,7 +69,7 @@ gst_vorbisenc_get_formats (GstPad *pad)
   };
   static const GstFormat sink_formats[] = {
     GST_FORMAT_BYTES,
-    GST_FORMAT_UNITS,
+    GST_FORMAT_DEFAULT,
     GST_FORMAT_TIME, 
     0
   };
@@ -82,10 +82,11 @@ gst_vorbisenc_get_formats (GstPad *pad)
 #define MIN_BITRATE_DEFAULT 	-1
 #define QUALITY_DEFAULT 	0.3
 
+static void             gst_vorbisenc_base_init         (gpointer g_class);
 static void 		gst_vorbisenc_class_init 	(VorbisEncClass *klass);
 static void 		gst_vorbisenc_init 		(VorbisEnc *vorbisenc);
 
-static void 		gst_vorbisenc_chain 		(GstPad *pad, GstBuffer *buf);
+static void 		gst_vorbisenc_chain 		(GstPad *pad, GstData *_data);
 static gboolean 	gst_vorbisenc_setup 		(VorbisEnc *vorbisenc);
 
 static void 		gst_vorbisenc_get_property 	(GObject *object, guint prop_id, 
@@ -106,7 +107,7 @@ vorbisenc_get_type (void)
   if (!vorbisenc_type) {
     static const GTypeInfo vorbisenc_info = {
       sizeof (VorbisEncClass), 
-      NULL,
+      gst_vorbisenc_base_init,
       NULL,
       (GClassInitFunc) gst_vorbisenc_class_init,
       NULL,
@@ -115,12 +116,59 @@ vorbisenc_get_type (void)
       0,
       (GInstanceInitFunc) gst_vorbisenc_init,
     };
-
+    static const GInterfaceInfo tag_setter_info = {
+      NULL,
+      NULL,
+      NULL
+    };
+    
     vorbisenc_type = g_type_register_static (GST_TYPE_ELEMENT, "VorbisEnc", &vorbisenc_info, 0);
+    
+    g_type_add_interface_static (vorbisenc_type, GST_TYPE_TAG_SETTER, &tag_setter_info);
   }
   return vorbisenc_type;
 }
 
+static GstCaps*
+vorbis_caps_factory (void)
+{
+  return gst_caps_new_simple ("application/ogg", NULL);
+}
+
+static GstCaps*
+raw_caps_factory (void)
+{
+  return
+   gst_caps_new_simple ("audio/x-raw-int",
+       "endianness", 	G_TYPE_INT, G_BYTE_ORDER,
+       "signed", 	G_TYPE_BOOLEAN, TRUE,
+       "width", 	G_TYPE_INT, 16,
+       "depth",    	G_TYPE_INT, 16,
+       "rate",     	GST_TYPE_INT_RANGE, 11025, 48000,
+       "channels", 	GST_TYPE_INT_RANGE, 1, 2,
+       NULL);
+}
+
+static void
+gst_vorbisenc_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  GstCaps *raw_caps, *vorbis_caps;
+
+  raw_caps = raw_caps_factory ();
+  vorbis_caps = vorbis_caps_factory ();
+
+  gst_vorbisenc_sink_template = gst_pad_template_new ("sink", GST_PAD_SINK, 
+		                                      GST_PAD_ALWAYS, 
+					              raw_caps);
+  gst_vorbisenc_src_template = gst_pad_template_new ("src", GST_PAD_SRC, 
+		                                     GST_PAD_ALWAYS, 
+					             vorbis_caps);
+  gst_element_class_add_pad_template (element_class, gst_vorbisenc_sink_template);
+  gst_element_class_add_pad_template (element_class, gst_vorbisenc_src_template);
+  gst_element_class_set_details (element_class, &vorbisenc_details);
+}
+
 static void
 gst_vorbisenc_class_init (VorbisEncClass * klass)
 {
@@ -145,13 +193,10 @@ gst_vorbisenc_class_init (VorbisEncClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_QUALITY, 
     g_param_spec_float ("quality", "Quality", 
 	    "Specify quality instead of specifying a particular bitrate.",
-	    -1.0, 10.0, QUALITY_DEFAULT, G_PARAM_READWRITE));
+	    0.0, 1.0, QUALITY_DEFAULT, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SERIAL, 
     g_param_spec_int ("serial", "Serial", "Specify a serial number for the stream. (-1 is random)",
 	    -1, G_MAXINT, -1, G_PARAM_READWRITE));
-  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_METADATA, 
-    g_param_spec_boxed ("metadata", "Metadata", "Metadata to add to the stream,",
-	    GST_TYPE_CAPS, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MANAGED, 
     g_param_spec_boolean ("managed", "Managed", "Enable bitrate management engine",
 	    FALSE, G_PARAM_READWRITE));
@@ -167,25 +212,24 @@ gst_vorbisenc_class_init (VorbisEncClass * klass)
   gstelement_class->change_state = gst_vorbisenc_change_state;
 }
 
-static GstPadConnectReturn
-gst_vorbisenc_sinkconnect (GstPad * pad, GstCaps * caps)
+static GstPadLinkReturn
+gst_vorbisenc_sinkconnect (GstPad * pad, const GstCaps * caps)
 {
   VorbisEnc *vorbisenc;
+  GstStructure *structure;
 
   vorbisenc = GST_VORBISENC (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_CONNECT_DELAYED;
-
-  gst_caps_get_int (caps, "channels", &vorbisenc->channels);
-  gst_caps_get_int (caps, "rate",     &vorbisenc->frequency);
+  structure = gst_caps_get_structure (caps, 0);
+  gst_structure_get_int  (structure, "channels", &vorbisenc->channels);
+  gst_structure_get_int  (structure, "rate",     &vorbisenc->frequency);
 
   gst_vorbisenc_setup (vorbisenc);
 
   if (vorbisenc->setup)
-    return GST_PAD_CONNECT_OK;
+    return GST_PAD_LINK_OK;
 
-  return GST_PAD_CONNECT_REFUSED;
+  return GST_PAD_LINK_REFUSED;
 }
 
 static gboolean
@@ -209,8 +253,6 @@ gst_vorbisenc_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
           *dest_value = src_value * GST_SECOND / avg;
           break;
@@ -220,8 +262,6 @@ gst_vorbisenc_convert_src (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
           *dest_value = src_value * avg / GST_SECOND;
           break;
@@ -251,13 +291,11 @@ gst_vorbisenc_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
   switch (src_format) {
     case GST_FORMAT_BYTES:
       switch (*dest_format) {
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  if (bytes_per_sample == 0)
             return FALSE;
 	  *dest_value = src_value / bytes_per_sample;
           break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	{
           gint byterate = bytes_per_sample * vorbisenc->frequency;
@@ -271,13 +309,11 @@ gst_vorbisenc_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
           res = FALSE;
       }
       break;
-    case GST_FORMAT_UNITS:
+    case GST_FORMAT_DEFAULT:
       switch (*dest_format) {
         case GST_FORMAT_BYTES:
 	  *dest_value = src_value * bytes_per_sample;
 	  break;
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_TIME;
         case GST_FORMAT_TIME:
 	  if (vorbisenc->frequency == 0)
             return FALSE;
@@ -289,12 +325,10 @@ gst_vorbisenc_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
       break;
     case GST_FORMAT_TIME:
       switch (*dest_format) {
-        case GST_FORMAT_DEFAULT:
-          *dest_format = GST_FORMAT_BYTES;
         case GST_FORMAT_BYTES:
 	  scale = bytes_per_sample;
 	  /* fallthrough */
-        case GST_FORMAT_UNITS:
+        case GST_FORMAT_DEFAULT:
 	  *dest_value = src_value * scale * vorbisenc->frequency / GST_SECOND;
           break;
         default:
@@ -307,19 +341,19 @@ gst_vorbisenc_convert_sink (GstPad *pad, GstFormat src_format, gint64 src_value,
   return res;
 }
 
-static const GstPadQueryType*
+static const GstQueryType*
 gst_vorbisenc_get_query_types (GstPad *pad)
 {
-  static const GstPadQueryType gst_vorbisenc_src_query_types[] = {
-    GST_PAD_QUERY_TOTAL,
-    GST_PAD_QUERY_POSITION,
+  static const GstQueryType gst_vorbisenc_src_query_types[] = {
+    GST_QUERY_TOTAL,
+    GST_QUERY_POSITION,
     0
   };
   return gst_vorbisenc_src_query_types;
 }
 
 static gboolean
-gst_vorbisenc_src_query (GstPad *pad, GstPadQueryType type,
+gst_vorbisenc_src_query (GstPad *pad, GstQueryType type,
 		   GstFormat *format, gint64 *value)
 {
   gboolean res = TRUE;
@@ -328,12 +362,9 @@ gst_vorbisenc_src_query (GstPad *pad, GstPadQueryType type,
   vorbisenc = GST_VORBISENC (gst_pad_get_parent (pad));
 
   switch (type) {
-    case GST_PAD_QUERY_TOTAL:
+    case GST_QUERY_TOTAL:
     {
       switch (*format) {
-	case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-	  /* fallthrough */
 	case GST_FORMAT_BYTES:
 	case GST_FORMAT_TIME:
         {
@@ -349,7 +380,7 @@ gst_vorbisenc_src_query (GstPad *pad, GstPadQueryType type,
 	    GstFormat peer_format = *peer_formats;
 
 	    /* do the probe */
-            if (gst_pad_query (GST_PAD_PEER (vorbisenc->sinkpad), GST_PAD_QUERY_TOTAL,
+            if (gst_pad_query (GST_PAD_PEER (vorbisenc->sinkpad), GST_QUERY_TOTAL,
 			       &peer_format, &peer_value)) 
 	    {
               GstFormat conv_format;
@@ -373,11 +404,8 @@ gst_vorbisenc_src_query (GstPad *pad, GstPadQueryType type,
       }
       break;
     }
-    case GST_PAD_QUERY_POSITION:
+    case GST_QUERY_POSITION:
       switch (*format) {
-	case GST_FORMAT_DEFAULT:
-          *format = GST_FORMAT_TIME;
-	  /* fall through */
 	default:
 	{
 	  /* we only know about our samples, convert to requested format */
@@ -401,7 +429,7 @@ gst_vorbisenc_init (VorbisEnc * vorbisenc)
   vorbisenc->sinkpad = gst_pad_new_from_template (gst_vorbisenc_sink_template, "sink");
   gst_element_add_pad (GST_ELEMENT (vorbisenc), vorbisenc->sinkpad);
   gst_pad_set_chain_function (vorbisenc->sinkpad, gst_vorbisenc_chain);
-  gst_pad_set_connect_function (vorbisenc->sinkpad, gst_vorbisenc_sinkconnect);
+  gst_pad_set_link_function (vorbisenc->sinkpad, gst_vorbisenc_sinkconnect);
   gst_pad_set_convert_function (vorbisenc->sinkpad, GST_DEBUG_FUNCPTR (gst_vorbisenc_convert_sink));
   gst_pad_set_formats_function (vorbisenc->sinkpad, GST_DEBUG_FUNCPTR (gst_vorbisenc_get_formats));
 
@@ -426,57 +454,109 @@ gst_vorbisenc_init (VorbisEnc * vorbisenc)
   
   vorbisenc->setup = FALSE;
   vorbisenc->eos = FALSE;
+  vorbisenc->header_sent = FALSE;
 
-  vorbisenc->metadata = GST_CAPS_NEW (
-		  "vorbisenc_metadata",
-                  "application/x-gst-metadata",
-                    "comment",  GST_PROPS_STRING ("Track encoded with GStreamer"),
-		    "date",     GST_PROPS_STRING (""),
-		    "tracknum", GST_PROPS_STRING (""),
-		    "title",    GST_PROPS_STRING (""),
-		    "artist",   GST_PROPS_STRING (""),
-		    "album",    GST_PROPS_STRING (""),
-		    "genre",    GST_PROPS_STRING ("")
-		  );
-
-
+  vorbisenc->tags = gst_tag_list_new ();
+  
   /* we're chained and we can deal with events */
   GST_FLAG_SET (vorbisenc, GST_ELEMENT_EVENT_AWARE);
 }
 
-static void 
-gst_vorbisenc_add_metadata (VorbisEnc *vorbisenc, GstCaps *caps)
+static void
+gst_vorbisenc_metadata_set1 (const GstTagList *list, const gchar *tag, gpointer vorbisenc)
 {
-  GList *props;
-  GstPropsEntry *prop;
+  gchar *vorbistag = NULL;
+  gchar *vorbisvalue = NULL;
+  guint i, count;
+  VorbisEnc *enc = GST_VORBISENC (vorbisenc);
 
-  if (caps == NULL)
-    return;
-
-  vorbis_comment_init (&vorbisenc->vc);
-
-  props = gst_caps_get_props (caps)->properties;
-  while (props) {
-    prop = (GstPropsEntry*)(props->data);
-    props = g_list_next(props);
-
-    if (gst_props_entry_get_type (prop) == GST_PROPS_STRING_TYPE) {
-      const gchar *name = gst_props_entry_get_name (prop);
-      const gchar *value;
-
-      gst_props_entry_get_string (prop, &value);
-
-      if (!value || strlen (value) == 0)
-	continue;
-
-      if (!strcmp (name, "comment")) {
-        vorbis_comment_add (&vorbisenc->vc, g_strdup (value));
-      }
-      else {
-        vorbis_comment_add_tag (&vorbisenc->vc, g_strdup (name), g_strdup (value));
+  count = gst_tag_list_get_tag_size (list, tag);
+  for (i = 0; i < count; i++) {
+    /* get tag name right */
+    if (strcmp (tag, GST_TAG_TITLE) == 0) {
+      vorbistag = g_strdup ("TITLE");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_VERSION) == 0) {
+      vorbistag = g_strdup ("VERSION");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_ALBUM) == 0) {
+      vorbistag = g_strdup ("ALBUM");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_TRACK_NUMBER) == 0) {
+      guint track_no;
+      vorbistag = g_strdup ("TRACKNUMBER");
+      g_assert (gst_tag_list_get_uint_index (list, tag, i, &track_no));
+      vorbisvalue = g_strdup_printf ("%u", track_no);
+    } else if (strcmp (tag, GST_TAG_ARTIST) == 0) {
+      vorbistag = g_strdup ("ARTIST");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_PERFORMER) == 0) {
+      vorbistag = g_strdup ("PERFORMER");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_COPYRIGHT) == 0) {
+      vorbistag = g_strdup ("COPYRIGHT");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_LICENSE) == 0) {
+      vorbistag = g_strdup ("LICENSE");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_ORGANIZATION) == 0) {
+      vorbistag = g_strdup ("ORGANIZATION");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_DESCRIPTION) == 0) {
+      vorbistag = g_strdup ("DESCRIPTION");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_GENRE) == 0) {
+      vorbistag = g_strdup ("GENRE");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_DATE) == 0) {
+      /* FIXME: how are dates represented in vorbis files? */
+      GDate *date;
+      guint u;
+      
+      vorbistag = g_strdup ("DATE");
+      g_assert (gst_tag_list_get_uint_index (list, tag, i, &u));
+      date = g_date_new_julian (u);
+      vorbisvalue = g_strdup_printf ("%04d-%02d-%02d", (gint) g_date_get_year (date),
+				   (gint) g_date_get_month (date), (gint) g_date_get_day (date));
+      g_date_free (date);
+    /* NOTE: GST_TAG_LOCATION != vorbis' location
+    } else if (strcmp (tag, PLACE) == 0) {
+      vorbistag = g_strdup ("LOCATION");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    */
+    } else if (strcmp (tag, GST_TAG_CONTACT) == 0) {
+      vorbistag = g_strdup ("CONTACT");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else if (strcmp (tag, GST_TAG_ISRC) == 0) {
+      vorbistag = g_strdup ("ISRC");
+      g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+    } else {
+      vorbistag = g_ascii_strup (tag, -1);
+      if (gst_tag_get_type (tag) == G_TYPE_STRING) {
+	g_assert (gst_tag_list_get_string_index (list, tag, i, &vorbisvalue));
+      } else {
+	const GValue *value = gst_tag_list_get_value_index (list, tag, i);
+        vorbisvalue = g_strdup_value_contents (value);
       }
     }
   }
+
+  vorbis_comment_add_tag (&enc->vc, vorbistag, vorbisvalue);
+}
+static void 
+gst_vorbisenc_set_metadata (VorbisEnc *vorbisenc)
+{
+  GstTagList *copy; 
+  const GstTagList *user_tags;
+  
+  user_tags = gst_tag_setter_get_list (GST_TAG_SETTER (vorbisenc));
+  if (!(vorbisenc->tags || user_tags))
+    return;
+
+  copy = gst_tag_list_merge (user_tags, vorbisenc->tags, gst_tag_setter_get_merge_mode (GST_TAG_SETTER (vorbisenc)));
+  vorbis_comment_init (&vorbisenc->vc);
+  gst_tag_list_foreach (copy, gst_vorbisenc_metadata_set1, vorbisenc);
+  gst_tag_list_free (copy);
 }
 
 static gchar*
@@ -608,8 +688,6 @@ gst_vorbisenc_setup (VorbisEnc *vorbisenc)
   }
   vorbis_encode_setup_init(&vorbisenc->vi);
 
-  gst_vorbisenc_add_metadata (vorbisenc, vorbisenc->metadata);
-
   /* set up the analysis state and auxiliary encoding storage */
   vorbis_analysis_init (&vorbisenc->vd, &vorbisenc->vi);
   vorbis_block_init (&vorbisenc->vd, &vorbisenc->vb);
@@ -627,28 +705,6 @@ gst_vorbisenc_setup (VorbisEnc *vorbisenc)
 
   ogg_stream_init (&vorbisenc->os, serial);
 
-  /* Vorbis streams begin with three headers; the initial header (with
-     most of the codec setup parameters) which is mandated by the Ogg
-     bitstream spec.  The second header holds any comment fields.  The
-     third header holds the bitstream codebook.  We merely need to
-     make the headers, then pass them to libvorbis one at a time;
-     libvorbis handles the additional Ogg bitstream constraints */
-
-  {
-    ogg_packet header;
-    ogg_packet header_comm;
-    ogg_packet header_code;
-
-    vorbis_analysis_headerout (&vorbisenc->vd, &vorbisenc->vc, &header, &header_comm, &header_code);
-    ogg_stream_packetin (&vorbisenc->os, &header);	/* automatically placed in its own
-							   page */
-    ogg_stream_packetin (&vorbisenc->os, &header_comm);
-    ogg_stream_packetin (&vorbisenc->os, &header_code);
-
-    /* no need to write out here.  We'll get to that in the main loop */
-    vorbisenc->flush_header = TRUE;
-  }
-
   vorbisenc->setup = TRUE;
 
   return TRUE;
@@ -668,13 +724,13 @@ gst_vorbisenc_write_page (VorbisEnc *vorbisenc, ogg_page *page)
 			             page->body,
 	        		     page->body_len);
 
-  GST_DEBUG (0, "vorbisenc: encoded buffer of %d bytes", 
+  GST_DEBUG ("vorbisenc: encoded buffer of %d bytes", 
 			GST_BUFFER_SIZE (outbuf));
 
   vorbisenc->bytes_out += GST_BUFFER_SIZE (outbuf);
 
   if (GST_PAD_IS_USABLE (vorbisenc->srcpad)) {
-    gst_pad_push (vorbisenc->srcpad, outbuf);
+    gst_pad_push (vorbisenc->srcpad, GST_DATA (outbuf));
   }
   else {
     gst_buffer_unref (outbuf);
@@ -682,8 +738,9 @@ gst_vorbisenc_write_page (VorbisEnc *vorbisenc, ogg_page *page)
 }
 
 static void
-gst_vorbisenc_chain (GstPad * pad, GstBuffer * buf)
+gst_vorbisenc_chain (GstPad * pad, GstData *_data)
 {
+  GstBuffer *buf = GST_BUFFER (_data);
   VorbisEnc *vorbisenc;
 
   g_return_if_fail (pad != NULL);
@@ -704,6 +761,15 @@ gst_vorbisenc_chain (GstPad * pad, GstBuffer * buf)
         vorbis_analysis_wrote (&vorbisenc->vd, 0);
 	gst_event_unref (event);
 	break;
+      case GST_EVENT_TAG:
+	if (vorbisenc->tags) {
+	  gst_tag_list_merge (vorbisenc->tags, gst_event_tag_get_list (event), 
+		  gst_tag_setter_get_merge_mode (GST_TAG_SETTER (vorbisenc)));
+	} else {
+	  g_assert_not_reached ();
+	}
+	gst_pad_event_default (pad, event);
+	return;
       default:
 	gst_pad_event_default (pad, event);
         return;
@@ -717,20 +783,32 @@ gst_vorbisenc_chain (GstPad * pad, GstBuffer * buf)
 
     if (!vorbisenc->setup) {
       gst_buffer_unref (buf);
-      gst_element_error (GST_ELEMENT (vorbisenc), "encoder not initialized (input is not audio?)");
+      GST_ELEMENT_ERROR (vorbisenc, CORE, NEGOTIATION, (NULL), ("encoder not initialized (input is not audio?)"));
       return;
     }
 
-    if (vorbisenc->flush_header) {
+    if (!vorbisenc->header_sent) {
       gint result;
+      /* Vorbis streams begin with three headers; the initial header (with
+	 most of the codec setup parameters) which is mandated by the Ogg
+	 bitstream spec.  The second header holds any comment fields.  The
+	 third header holds the bitstream codebook.  We merely need to
+	 make the headers, then pass them to libvorbis one at a time;
+	 libvorbis handles the additional Ogg bitstream constraints */
+      ogg_packet header;
+      ogg_packet header_comm;
+      ogg_packet header_code;
+
+      gst_vorbisenc_set_metadata (vorbisenc);
+      vorbis_analysis_headerout (&vorbisenc->vd, &vorbisenc->vc, &header, &header_comm, &header_code);
+      ogg_stream_packetin (&vorbisenc->os, &header); /* automatically placed in its own page */
+      ogg_stream_packetin (&vorbisenc->os, &header_comm);
+      ogg_stream_packetin (&vorbisenc->os, &header_code);
 
       while ((result = ogg_stream_flush(&vorbisenc->os, &vorbisenc->og))) {
-        if (!result) 
-          break;
-
 	gst_vorbisenc_write_page (vorbisenc, &vorbisenc->og);
       }
-      vorbisenc->flush_header = FALSE;
+      vorbisenc->header_sent = TRUE;
     }
   
     /* data to encode */
@@ -793,7 +871,7 @@ gst_vorbisenc_chain (GstPad * pad, GstBuffer * buf)
     vorbis_block_clear (&vorbisenc->vb);
     vorbis_dsp_clear (&vorbisenc->vd);
     vorbis_info_clear (&vorbisenc->vi);
-    gst_pad_push (vorbisenc->srcpad, GST_BUFFER (gst_event_new (GST_EVENT_EOS)));
+    gst_pad_push (vorbisenc->srcpad, GST_DATA (gst_event_new (GST_EVENT_EOS)));
     gst_element_set_eos (GST_ELEMENT (vorbisenc));
   }
 }
@@ -824,9 +902,6 @@ gst_vorbisenc_get_property (GObject * object, guint prop_id, GValue * value, GPa
     case ARG_SERIAL:
       g_value_set_int (value, vorbisenc->serial);
       break;
-    case ARG_METADATA:
-      g_value_set_static_boxed (value, vorbisenc->metadata);
-      break;
     case ARG_MANAGED:
       g_value_set_boolean (value, vorbisenc->managed);
       break;
@@ -892,9 +967,6 @@ gst_vorbisenc_set_property (GObject * object, guint prop_id, const GValue * valu
     case ARG_SERIAL:
       vorbisenc->serial = g_value_get_int (value);
       break;
-    case ARG_METADATA:
-      vorbisenc->metadata = g_value_get_boxed (value);
-      break;
     case ARG_MANAGED:
       vorbisenc->managed = g_value_get_boolean (value);
       break;
@@ -919,6 +991,9 @@ gst_vorbisenc_change_state (GstElement *element)
       break;
     case GST_STATE_PAUSED_TO_READY:
       vorbisenc->setup = FALSE;
+      vorbisenc->header_sent = FALSE;
+      gst_tag_list_free (vorbisenc->tags);
+      vorbisenc->tags = gst_tag_list_new ();
       break;
     case GST_STATE_READY_TO_NULL:
     default:
