@@ -36,10 +36,6 @@ enum {
 
 static void     gst_riff_read_class_init   (GstRiffReadClass *klass);
 static void     gst_riff_read_init         (GstRiffRead *riff);
-static void     gst_riff_read_get_property (GObject     *object,
-					    guint        prop_id, 	
-					    GValue      *value,
-					    GParamSpec  *pspec);
 
 static GstElementStateReturn
 		gst_riff_read_change_state (GstElement  *element);
@@ -75,17 +71,10 @@ gst_riff_read_get_type (void)
 static void
 gst_riff_read_class_init (GstRiffReadClass *klass) 
 {
-  GObjectClass *gobject_class = G_OBJECT_CLASS (klass);
   GstElementClass *gstelement_class = GST_ELEMENT_CLASS (klass);
 
-  g_object_class_install_property (gobject_class, ARG_METADATA,
-    g_param_spec_boxed ("metadata", "Metadata", "Metadata",
-                        GST_TYPE_CAPS, G_PARAM_READABLE));
-
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
   
-  gobject_class->get_property = gst_riff_read_get_property;
-
   gstelement_class->change_state = gst_riff_read_change_state;
 }
 
@@ -95,25 +84,6 @@ gst_riff_read_init (GstRiffRead *riff)
   riff->sinkpad = NULL;
   riff->bs = NULL;
   riff->level = NULL;
-  riff->metadata = NULL;
-}
-
-static void
-gst_riff_read_get_property (GObject    *object,
-			    guint       prop_id, 	
-			    GValue     *value,
-			    GParamSpec *pspec)
-{
-  GstRiffRead *riff = GST_RIFF_READ (object);
-
-  switch (prop_id) {
-    case ARG_METADATA:
-      g_value_set_boxed (value, riff->metadata);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
 }
 
 static GstElementStateReturn
@@ -128,7 +98,6 @@ gst_riff_read_change_state (GstElement *element)
       riff->bs = gst_bytestream_new (riff->sinkpad);
       break;
     case GST_STATE_PAUSED_TO_READY:
-      gst_caps_replace (&riff->metadata, NULL);
       gst_bytestream_destroy (riff->bs);
       while (riff->level) {
         GstRiffLevel *level = riff->level->data;
@@ -189,19 +158,19 @@ gst_riff_peek_head (GstRiffRead *riff,
   guint8 *data;
 
   /* read */
-  if (gst_bytestream_peek_bytes (riff->bs, &data, 8) != 8) {
+  while (gst_bytestream_peek_bytes (riff->bs, &data, 8) != 8) {
     GstEvent *event = NULL;
     guint32 remaining;
 
     /* Here, we might encounter EOS */
     gst_bytestream_get_status (riff->bs, &remaining, &event);
-    if (event && GST_EVENT_TYPE (event) == GST_EVENT_EOS) {
+    if (event) {
       gst_pad_event_default (riff->sinkpad, event);
     } else {
       gst_event_unref (event);
-      gst_element_error (GST_ELEMENT (riff), "Read error");
+      GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
+      return FALSE;
     }
-    return FALSE;
   }
 
   /* parse tag + length (if wanted) */
@@ -228,7 +197,7 @@ gst_riff_read_element_data (GstRiffRead *riff,
   GstBuffer *buf = NULL;
 
   if (gst_bytestream_peek (riff->bs, &buf, length) != length) {
-    gst_element_error (GST_ELEMENT (riff), "Read error");
+    GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
     if (buf)
       gst_buffer_unref (buf);
     return NULL;
@@ -253,7 +222,7 @@ gst_riff_read_seek (GstRiffRead *riff,
 {
   guint64 length = gst_bytestream_length (riff->bs);
   guint32 remaining;
-  GstEvent *event;
+  GstEvent *event = NULL;
   guchar *data;
 
   /* hack for AVI files with broken idx1 size chunk markers */
@@ -271,23 +240,27 @@ gst_riff_read_seek (GstRiffRead *riff,
 
   /* now seek */
   if (!gst_bytestream_seek (riff->bs, offset, GST_SEEK_METHOD_SET)) {
-    gst_element_error (GST_ELEMENT (riff), "Seek failed");
+    GST_ELEMENT_ERROR (riff, RESOURCE, SEEK, (NULL), (NULL));
     return NULL;
   }
 
   /* and now, peek a new byte. This will fail because there's a
    * pending event. Then, take the event and return it. */
-  if (gst_bytestream_peek_bytes (riff->bs, &data, 1))
-    g_warning ("Unexpected data after seek");
+  while (!event) {
+    if (gst_bytestream_peek_bytes (riff->bs, &data, 1)) {
+      GST_WARNING ("Unexpected data after seek - this means seek failed");
+      break;
+    }
 
-  /* get the discont event and return */
-  gst_bytestream_get_status (riff->bs, &remaining, &event);
-  if (!event || GST_EVENT_TYPE (event) != GST_EVENT_DISCONTINUOUS) {
-    gst_element_error (GST_ELEMENT (riff),
-		       "No discontinuity event after seek");
-    if (event)
-      gst_event_unref (event);
-    return NULL;
+    /* get the discont event and return */
+    gst_bytestream_get_status (riff->bs, &remaining, &event);
+    if (!event) {
+      GST_WARNING ("No discontinuity event after seek - seek failed");
+      break;
+    } else if (GST_EVENT_TYPE (event) != GST_EVENT_DISCONTINUOUS) {
+      gst_pad_event_default (riff->sinkpad, event);
+      event = NULL;
+    }
   }
 
   return event;
@@ -327,7 +300,7 @@ gst_riff_peek_list (GstRiffRead *riff)
   }
 
   if (gst_bytestream_peek_bytes (riff->bs, &data, 12) != 12) {
-    gst_element_error (GST_ELEMENT (riff), "Read error");
+    GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
     return 0;
   }
 
@@ -687,7 +660,7 @@ gst_riff_read_list (GstRiffRead *riff,
   }
   gst_bytestream_flush_fast (riff->bs, 8);
   if (gst_bytestream_peek_bytes (riff->bs, &data, 4) != 4) {
-    gst_element_error (GST_ELEMENT (riff), "Read error");
+    GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
     return FALSE;
   }
   gst_bytestream_flush_fast (riff->bs, 4);
@@ -714,7 +687,8 @@ gst_riff_read_info (GstRiffRead *riff)
   GstRiffLevel *level;
   GList *last;
   gchar *name, *type;
-  GstProps *props;
+  GstTagList *taglist;
+  gboolean have_tags = FALSE;
 
   /* What we're doing here is ugly (oh no!); we look
    * at our LIST tag size and assure that we do not
@@ -726,84 +700,83 @@ gst_riff_read_info (GstRiffRead *riff)
   end = level->start + level->length;
   g_free (level);
 
-  props = gst_props_empty_new ();
+  taglist = gst_tag_list_new ();
 
   while (gst_bytestream_tell (riff->bs) < end) {
     if (!gst_riff_peek_head (riff, &tag, NULL, NULL)) {
-      gst_props_unref (props);
       return FALSE;
     }
 
     /* find out the type of metadata */
     switch (tag) {
       case GST_RIFF_INFO_IARL:
-        type = "Location";
+        type = GST_TAG_LOCATION;
         break;
       case GST_RIFF_INFO_IART:
-        type = "Artist";
+        type = GST_TAG_ARTIST;
         break;
       case GST_RIFF_INFO_ICMS:
-        type = "Commissioner";
+        type = NULL; /*"Commissioner";*/
         break;
       case GST_RIFF_INFO_ICMT:
-        type = "Comment";
+        type = GST_TAG_COMMENT;
         break;
       case GST_RIFF_INFO_ICOP:
-        type = "Copyright";
+        type = GST_TAG_COPYRIGHT;
         break;
       case GST_RIFF_INFO_ICRD:
-        type = "Creation Date";
+        type = GST_TAG_DATE;
         break;
       case GST_RIFF_INFO_ICRP:
-        type = "Cropped";
+        type = NULL; /*"Cropped";*/
         break;
       case GST_RIFF_INFO_IDIM:
-        type = "Dimensions";
+        type = NULL; /*"Dimensions";*/
         break;
       case GST_RIFF_INFO_IDPI:
-        type = "Dots per Inch";
+        type = NULL; /*"Dots per Inch";*/
         break;
       case GST_RIFF_INFO_IENG:
-        type = "Engineer";
+        type = NULL; /*"Engineer";*/
         break;
       case GST_RIFF_INFO_IGNR:
-        type = "Genre";
+        type = GST_TAG_GENRE;
         break;
       case GST_RIFF_INFO_IKEY:
-        type = "Keywords";
+        type = NULL; /*"Keywords";*/;
         break;
       case GST_RIFF_INFO_ILGT:
-        type = "Lightness";
+        type = NULL; /*"Lightness";*/
         break;
       case GST_RIFF_INFO_IMED:
-        type = "Medium";
+        type = NULL; /*"Medium";*/
         break;
       case GST_RIFF_INFO_INAM:
-        type = "Title"; /* "Name" */
+        type = GST_TAG_TITLE;
         break;
       case GST_RIFF_INFO_IPLT:
-        type = "Palette";
+        type = NULL; /*"Palette";*/
         break;
       case GST_RIFF_INFO_IPRD:
-        type = "Product";
+        type = NULL; /*"Product";*/
         break;
       case GST_RIFF_INFO_ISBJ:
-        type = "Subject";
+        type = NULL; /*"Subject";*/
         break;
       case GST_RIFF_INFO_ISFT:
-        type = "Encoder"; /* "Software" */
+        type = GST_TAG_ENCODER;
         break;
       case GST_RIFF_INFO_ISHP:
-        type = "Sharpness";
+        type = NULL; /*"Sharpness";*/
         break;
       case GST_RIFF_INFO_ISRC:
-        type = "Source";
+        type = GST_TAG_ISRC;
         break;
       case GST_RIFF_INFO_ISRF:
-        type = "Source Form";
+        type = NULL; /*"Source Form";*/
         break;
       case GST_RIFF_INFO_ITCH:
-        type = "Technician";
+        type = NULL; /*"Technician";*/
         break;
       default:
         type = NULL;
@@ -813,27 +786,48 @@ gst_riff_read_info (GstRiffRead *riff)
     }
 
     if (type) {
-      GstPropsEntry *entry;
-
+      name = NULL;
       if (!gst_riff_read_ascii (riff, &tag, &name)) {
-        gst_props_unref (props);
         return FALSE;
       }
 
-      entry = gst_props_entry_new (type, GST_PROPS_STRING (name));
-      gst_props_add_entry (props, entry);
+      if (name && name[0] != '\0') {
+        GValue src = { 0 }, dest = { 0 };
+        GType dest_type = gst_tag_get_type (type);
+
+        have_tags = TRUE;
+        g_value_init (&src, G_TYPE_STRING);
+        g_value_set_string (&src, name);
+        g_value_init (&dest, dest_type);
+        g_value_transform (&src, &dest);
+        g_value_unset (&src);
+        gst_tag_list_add_values (taglist, GST_TAG_MERGE_APPEND,
+				 type, &dest, NULL);
+        g_value_unset (&dest);
+      }
+      g_free (name);
     } else {
       gst_riff_read_skip (riff);
     }
   }
 
-  /* let the world know about this wonderful thing */
-  gst_props_debug (props);
-  gst_caps_replace_sink (&riff->metadata,
-			 gst_caps_new ("riff_metadata",
-				       "application/x-gst-metadata",
-				       props));
-  g_object_notify (G_OBJECT (riff), "metadata");
+  if (have_tags) {
+    GstElement *element = GST_ELEMENT (riff);
+    GstEvent *event = gst_event_new_tag (taglist);
+    const GList *padlist;
+
+    /* let the world know about this wonderful thing */
+    for (padlist = gst_element_get_pad_list (element);
+	 padlist != NULL; padlist = padlist->next) {
+      if (GST_PAD_IS_SRC (padlist->data) && GST_PAD_IS_USABLE(padlist->data)) {
+        gst_event_ref (event);
+        gst_pad_push (GST_PAD (padlist->data), GST_DATA (event));
+      }
+    }
+    gst_event_unref (event);
+    gst_element_found_tags (GST_ELEMENT (riff), taglist);
+  }
+  gst_tag_list_free (taglist);
 
   return TRUE;
 }
@@ -854,14 +848,14 @@ gst_riff_read_header (GstRiffRead *riff,
   if (!gst_riff_peek_head (riff, &tag, &length, NULL))
     return FALSE;
   if (tag != GST_RIFF_TAG_RIFF) {
-    GST_WARNING ("Not a RIFF file");
+    GST_ELEMENT_ERROR (riff, STREAM, WRONG_TYPE, (NULL), (NULL));
     return FALSE;
   }
   gst_bytestream_flush_fast (riff->bs, 8);
 
   /* doctype */
   if (gst_bytestream_peek_bytes (riff->bs, &data, 4) != 4) {
-    gst_element_error (GST_ELEMENT (riff), "Read error");
+    GST_ELEMENT_ERROR (riff, RESOURCE, READ, (NULL), (NULL));
     return FALSE;
   }
   gst_bytestream_flush_fast (riff->bs, 4);
