@@ -16,6 +16,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
+/* Element-Checklist-Version: 5 */
 
 
 #ifdef HAVE_CONFIG_H
@@ -30,15 +31,12 @@
 #include <gst/resample/resample.h>
 
 /* elementfactory information */
-static GstElementDetails audioscale_details = {
+static GstElementDetails gst_audioscale_details = GST_ELEMENT_DETAILS (
   "Audio scaler",
-  "Filter/Audio",
-  "LGPL",
-  "Audio resampler",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2000",
-};
+  "Filter/Converter/Audio",
+  "Resample audio",
+  "David Schleef <ds@schleef.org>"
+);
 
 /* Audioscale signals and args */
 enum {
@@ -53,22 +51,20 @@ enum {
   /* FILL ME */
 };
 
-GST_PAD_TEMPLATE_FACTORY (sink_factory,
+static GstStaticPadTemplate gst_audioscale_sink_template =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  gst_caps_new ("audioscale_sink",
-		"audio/x-raw-int",
-		  GST_AUDIO_INT_PAD_TEMPLATE_PROPS)
+  GST_STATIC_CAPS ( GST_AUDIO_INT_PAD_TEMPLATE_CAPS)
 );
 
-GST_PAD_TEMPLATE_FACTORY (src_factory,
+static GstStaticPadTemplate gst_audioscale_src_template =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  gst_caps_new ("audioscale_src",
-		"audio/x-raw-int",
-		  GST_AUDIO_INT_PAD_TEMPLATE_PROPS)
+  GST_STATIC_CAPS ( GST_AUDIO_INT_PAD_TEMPLATE_CAPS)
 );
 
 #define GST_TYPE_AUDIOSCALE_METHOD (gst_audioscale_method_get_type())
@@ -89,6 +85,7 @@ gst_audioscale_method_get_type (void)
   return audioscale_method_type;
 }
 
+static void     gst_audioscale_base_init        (gpointer g_class);
 static void	gst_audioscale_class_init	(AudioscaleClass *klass);
 static void	gst_audioscale_init		(Audioscale *audioscale);
 
@@ -110,7 +107,8 @@ audioscale_get_type (void)
 
   if (!audioscale_type) {
     static const GTypeInfo audioscale_info = {
-      sizeof(AudioscaleClass),      NULL,
+      sizeof(AudioscaleClass),
+      gst_audioscale_base_init,
       NULL,
       (GClassInitFunc)gst_audioscale_class_init,
       NULL,
@@ -124,6 +122,19 @@ audioscale_get_type (void)
   return audioscale_type;
 }
 
+static void
+gst_audioscale_base_init (gpointer g_class)
+{
+  GstElementClass *gstelement_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_audioscale_src_template));
+  gst_element_class_add_pad_template (gstelement_class,
+      gst_static_pad_template_get (&gst_audioscale_sink_template));
+
+  gst_element_class_set_details (gstelement_class, &gst_audioscale_details);
+}
+
 static void
 gst_audioscale_class_init (AudioscaleClass *klass)
 {
@@ -133,133 +144,124 @@ gst_audioscale_class_init (AudioscaleClass *klass)
   gobject_class = (GObjectClass*)klass;
   gstelement_class = (GstElementClass*)klass;
 
+  gobject_class->set_property = gst_audioscale_set_property;
+  gobject_class->get_property = gst_audioscale_get_property;
+
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_FILTERLEN,
 	g_param_spec_int ("filter_length", "filter_length", "filter_length",
                           0, G_MAXINT, 16, G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
+
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_METHOD,
 	g_param_spec_enum ("method", "method", "method", GST_TYPE_AUDIOSCALE_METHOD,
                            RESAMPLE_SINC, G_PARAM_READWRITE|G_PARAM_CONSTRUCT));
 
   parent_class = g_type_class_ref(GST_TYPE_ELEMENT);
-
-  gobject_class->set_property = gst_audioscale_set_property;
-  gobject_class->get_property = gst_audioscale_get_property;
-
 }
 
 static GstCaps *
-gst_audioscale_getcaps (GstPad *pad, GstCaps *caps)
+gst_audioscale_getcaps (GstPad *pad)
 {
   Audioscale *audioscale;
-  GstCaps *peercaps;
+  GstCaps *caps;
+  GstPad *otherpad;
+  int i;
 
   audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
 
-  if (pad == audioscale->srcpad){
-    peercaps = gst_pad_get_allowed_caps (audioscale->sinkpad);
-  }else{
-    peercaps = gst_pad_get_allowed_caps (audioscale->srcpad);
-  }
+  otherpad = (pad == audioscale->srcpad) ? audioscale->sinkpad :
+    audioscale->srcpad;
+  caps = gst_pad_get_allowed_caps (otherpad);
 
-  if(peercaps == GST_CAPS_NONE){
-    return GST_CAPS_NONE;
-  }
-
-  caps = gst_caps_copy (peercaps);
-#if 1
   /* we do this hack, because the audioscale lib doesn't handle
    * rate conversions larger than a factor of 2 */
-  if(gst_caps_has_property_typed(caps, "rate", GST_PROPS_INT_RANGE_TYPE)){
+  for (i=0;i<gst_caps_get_size(caps);i++){
     int rate_min, rate_max;
+    GstStructure *structure = gst_caps_get_structure (caps, i);
+    const GValue *value;
 
-    gst_props_entry_get_int_range (gst_props_get_entry(caps->properties, "rate"),
-	&rate_min, &rate_max);
-    gst_caps_set (caps, "rate", GST_PROPS_INT_RANGE((rate_min+1)/2,
-	  rate_max*2));
-  }else{
-    int rate;
+    value = gst_structure_get_value (structure, "rate");
+    if (value == NULL) return NULL;
 
-    gst_caps_get_int (caps, "rate", &rate);
-    gst_caps_set (caps, "rate", GST_PROPS_INT_RANGE((rate+1)/2,rate*2));
+    if (G_VALUE_TYPE (value) == G_TYPE_INT) {
+      rate_min = g_value_get_int (value);
+      rate_max = rate_min;
+    } else if (G_VALUE_TYPE (value) == GST_TYPE_INT_RANGE) {
+      rate_min = gst_value_get_int_range_min (value);
+      rate_max = gst_value_get_int_range_max (value);
+    } else {
+      return NULL;
+    }
+
+    rate_min /= 2;
+    if (rate_max < G_MAXINT/2){
+      rate_max *=2;
+    } else {
+      rate_max = G_MAXINT;
+    }
+
+    gst_structure_set (structure, "rate", GST_TYPE_INT_RANGE, rate_min,
+	rate_max, NULL);
   }
-#else
-  gst_caps_set (caps, "rate", GST_PROPS_INT_RANGE(4000,96000));
-#endif
 
   return caps;
 }
 
 static GstPadLinkReturn
-gst_audioscale_sink_link (GstPad * pad, GstCaps * caps)
+gst_audioscale_link (GstPad * pad, const GstCaps * caps)
 {
   Audioscale *audioscale;
   resample_t *r;
-  GstCaps *caps1;
-  GstCaps *caps2;
-  GstCaps *peercaps;
-  gint rate;
+  GstStructure *structure;
+  int rate;
+  int channels;
   int ret;
+  GstPadLinkReturn link_ret;
+  GstPad *otherpad;
 
   audioscale = GST_AUDIOSCALE (gst_pad_get_parent (pad));
   r = audioscale->resample;
 
-  if (!GST_CAPS_IS_FIXED (caps)){
-    return GST_PAD_LINK_DELAYED;
-  }
+  otherpad = (pad == audioscale->srcpad) ? audioscale->sinkpad
+    : audioscale->srcpad;
 
-  ret = gst_pad_try_set_caps (audioscale->srcpad, caps);
+  structure = gst_caps_get_structure (caps, 0);
 
-  if(ret == GST_PAD_LINK_OK || ret == GST_PAD_LINK_DONE){
+  ret = gst_structure_get_int (structure, "rate", &rate);
+  ret &= gst_structure_get_int (structure, "channels", &channels);
+
+  link_ret = gst_pad_try_set_caps (otherpad, gst_caps_copy (caps));
+  if (GST_PAD_LINK_SUCCESSFUL (link_ret)){
     audioscale->passthru = TRUE;
-    return ret;
+    r->channels = channels;
+    r->i_rate = rate;
+    r->o_rate = rate;
+    return link_ret;
   }
-
   audioscale->passthru = FALSE;
 
-  gst_caps_get_int (caps, "rate",     &rate);
-  gst_caps_get_int (caps, "channels", &r->channels);
 
-  r->i_rate = rate;
+  if (gst_pad_is_negotiated (otherpad)) {
+    GstCaps *trycaps = gst_caps_copy (caps);
+
+    gst_caps_set_simple (trycaps,
+        "rate", G_TYPE_INT,
+        (int)((pad == audioscale->srcpad) ? r->i_rate : r->o_rate),
+        NULL);
+    link_ret = gst_pad_try_set_caps (otherpad, trycaps);
+    if (GST_PAD_LINK_FAILED (link_ret)){
+      return link_ret;
+    }
+  }
+
+  r->channels = channels;
+  if (pad == audioscale->srcpad) {
+    r->o_rate = rate;
+  } else {
+    r->i_rate = rate;
+  }
   resample_reinit(r);
 
-  peercaps = gst_pad_get_allowed_caps (audioscale->srcpad);
-
-  caps1 = gst_caps_copy (caps);
-#if 1
-  /* we do this hack, because the audioscale lib doesn't handle
-   * rate conversions larger than a factor of 2 */
-  if(gst_caps_has_property_typed(caps1, "rate", GST_PROPS_INT_RANGE_TYPE)){
-    int rate_min, rate_max;
-
-    gst_props_entry_get_int_range (gst_props_get_entry(caps1->properties, "rate"),
-	&rate_min, &rate_max);
-    gst_caps_set (caps1, "rate", GST_PROPS_INT_RANGE((rate_min+1)/2,
-	  rate_max*2));
-  }else{
-    gst_caps_get_int (caps1, "rate", &rate);
-    gst_caps_set (caps1, "rate", GST_PROPS_INT_RANGE((rate+1)/2,rate*2));
-  }
-#else
-  gst_caps_set (caps1, "rate", GST_PROPS_INT_RANGE(4000,96000));
-#endif
-  caps2 = gst_caps_intersect(caps1, peercaps);
-  gst_caps_unref(caps1);
-
-  if(caps2 == GST_CAPS_NONE){
-    return GST_PAD_LINK_REFUSED;
-  }
-
-  if (GST_CAPS_IS_FIXED (caps2)) {
-    ret = gst_pad_try_set_caps (audioscale->srcpad, caps2);
-    gst_caps_get_int (caps, "rate",     &rate);
-    r->o_rate = rate;
-    audioscale->targetfrequency = rate;
-    resample_reinit(r);
-    return ret;
-  }
-  
-  gst_caps_unref (caps2);
-  return GST_PAD_LINK_DELAYED;
+  return GST_PAD_LINK_OK;
 }
 
 static void *
@@ -270,7 +272,7 @@ gst_audioscale_get_buffer (void *priv, unsigned int size)
   audioscale->outbuf = gst_buffer_new();
   GST_BUFFER_SIZE(audioscale->outbuf) = size;
   GST_BUFFER_DATA(audioscale->outbuf) = g_malloc(size);
-  GST_BUFFER_TIMESTAMP(audioscale->outbuf) = audioscale->offset * GST_SECOND / audioscale->targetfrequency;
+  GST_BUFFER_TIMESTAMP(audioscale->outbuf) = audioscale->offset * GST_SECOND / audioscale->resample->o_rate;
   audioscale->offset += size / sizeof(gint16) / audioscale->resample->channels;
 
   return GST_BUFFER_DATA(audioscale->outbuf);
@@ -282,16 +284,17 @@ gst_audioscale_init (Audioscale *audioscale)
   resample_t *r;
 
   audioscale->sinkpad = gst_pad_new_from_template (
-	GST_PAD_TEMPLATE_GET (sink_factory), "sink");
+      gst_static_pad_template_get (&gst_audioscale_sink_template), "sink");
   gst_element_add_pad(GST_ELEMENT(audioscale),audioscale->sinkpad);
   gst_pad_set_chain_function(audioscale->sinkpad,gst_audioscale_chain);
-  gst_pad_set_link_function (audioscale->sinkpad, gst_audioscale_sink_link);
+  gst_pad_set_link_function (audioscale->sinkpad, gst_audioscale_link);
   gst_pad_set_getcaps_function (audioscale->sinkpad, gst_audioscale_getcaps);
 
   audioscale->srcpad = gst_pad_new_from_template (
-	GST_PAD_TEMPLATE_GET (src_factory), "src");
+      gst_static_pad_template_get (&gst_audioscale_src_template), "src");
 
   gst_element_add_pad(GST_ELEMENT(audioscale),audioscale->srcpad);
+  gst_pad_set_link_function (audioscale->srcpad, gst_audioscale_link);
   gst_pad_set_getcaps_function (audioscale->srcpad, gst_audioscale_getcaps);
 
   r = g_new0(resample_t,1);
@@ -333,10 +336,8 @@ gst_audioscale_chain (GstPad *pad, GstData *_data)
   data = GST_BUFFER_DATA(buf);
   size = GST_BUFFER_SIZE(buf);
 
-  GST_DEBUG (
-	     "gst_audioscale_chain: got buffer of %ld bytes in '%s'\n",
-	     size, gst_element_get_name (GST_ELEMENT (audioscale)));
-
+  GST_DEBUG ("gst_audioscale_chain: got buffer of %ld bytes in '%s'\n",
+      size, gst_element_get_name (GST_ELEMENT (audioscale)));
 
   resample_scale (audioscale->resample, data, size);
 
@@ -397,29 +398,29 @@ gst_audioscale_get_property (GObject *object, guint prop_id, GValue *value, GPar
 
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  /* create an elementfactory for the audioscale element */
-  factory = gst_element_factory_new ("audioscale", GST_TYPE_AUDIOSCALE, &audioscale_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (src_factory));
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (sink_factory));
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
   /* load support library */
   if (!gst_library_load ("gstresample"))
     return FALSE;
 
+  if (!gst_element_register (plugin, "audioscale", GST_RANK_NONE,
+        GST_TYPE_AUDIOSCALE)) {
+    return FALSE;
+  }
+
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "audioscale",
-  plugin_init
-};
+  "Resamples audio",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN
+)
+
