@@ -23,6 +23,8 @@
 #include "config.h"
 #endif
 
+#include "gst/gst-i18n-plugin.h"
+
 #include "gstavidemux.h"
 #include "gstavimux.h"
 
@@ -32,6 +34,11 @@ plugin_init (GstPlugin *plugin)
   if (!gst_library_load ("riff"))
     return FALSE;
 
+#ifdef ENABLE_NLS
+  setlocale (LC_ALL, "");
+  bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
+#endif /* ENABLE_NLS */
+
   return (gst_element_register (plugin, "avidemux",
 				GST_RANK_PRIMARY,
 				GST_TYPE_AVI_DEMUX) &&
