@@ -67,15 +67,12 @@ struct _GstShagadelicTVClass
 };
 
 /* elementfactory information */
-GstElementDetails gst_shagadelictv_details = {
+static GstElementDetails gst_shagadelictv_details = GST_ELEMENT_DETAILS (
   "ShagadelicTV",
-  "Filter/Video/Effect",
-  "LGPL",
+  "Filter/Effect/Video",
   "Oh behave, ShagedelicTV makes images shagadelic!",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001 FUKUCHI Kentarou",
-};
+  "Wim Taymans <wim.taymans@chello.be>"
+);
 
 
 /* Filter signals and args */
@@ -90,6 +87,7 @@ enum
   ARG_0,
 };
 
+static void	gst_shagadelictv_base_init	(gpointer g_class);
 static void 	gst_shagadelictv_class_init 	(GstShagadelicTVClass * klass);
 static void 	gst_shagadelictv_init 		(GstShagadelicTV * filter);
 
@@ -111,7 +109,8 @@ GType gst_shagadelictv_get_type (void)
 
   if (!shagadelictv_type) {
     static const GTypeInfo shagadelictv_info = {
-      sizeof (GstShagadelicTVClass), NULL,
+      sizeof (GstShagadelicTVClass), 
+      gst_shagadelictv_base_init,
       NULL,
       (GClassInitFunc) gst_shagadelictv_class_init,
       NULL,
@@ -126,6 +125,19 @@ GType gst_shagadelictv_get_type (void)
   return shagadelictv_type;
 }
 
+static void
+gst_shagadelictv_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get(&gst_effectv_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get(&gst_effectv_sink_template));
+ 
+  gst_element_class_set_details (element_class, &gst_shagadelictv_details);
+}
+
 static void
 gst_shagadelictv_class_init (GstShagadelicTVClass * klass)
 {
@@ -142,18 +154,18 @@ gst_shagadelictv_class_init (GstShagadelicTVClass * klass)
 }
 
 static GstPadLinkReturn
-gst_shagadelictv_sinkconnect (GstPad * pad, GstCaps * caps)
+gst_shagadelictv_sinkconnect (GstPad * pad, const GstCaps * caps)
 {
   GstShagadelicTV *filter;
   gint area;
+  GstStructure *structure;
 
   filter = GST_SHAGADELICTV (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
+  structure = gst_caps_get_structure (caps, 0);
 
-  gst_caps_get_int (caps, "width", &filter->width);
-  gst_caps_get_int (caps, "height", &filter->height);
+  gst_structure_get_int  (structure, "width", &filter->width);
+  gst_structure_get_int  (structure, "height", &filter->height);
 
   area = filter->width * filter->height;
 
@@ -165,18 +177,20 @@ gst_shagadelictv_sinkconnect (GstPad * pad, GstCaps * caps)
 
   gst_shagadelic_initialize (filter);
 
-  return gst_pad_try_set_caps (filter->srcpad, gst_caps_ref (caps));
+  return gst_pad_try_set_caps (filter->srcpad, caps);
 }
 
 static void
 gst_shagadelictv_init (GstShagadelicTV * filter)
 {
-  filter->sinkpad = gst_pad_new_from_template (gst_effectv_sink_factory (), "sink");
+  filter->sinkpad = gst_pad_new_from_template (
+      gst_static_pad_template_get(&gst_effectv_sink_template), "sink");
   gst_pad_set_chain_function (filter->sinkpad, gst_shagadelictv_chain);
   gst_pad_set_link_function (filter->sinkpad, gst_shagadelictv_sinkconnect);
   gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
 
-  filter->srcpad = gst_pad_new_from_template (gst_effectv_src_factory (), "src");
+  filter->srcpad = gst_pad_new_from_template (
+      gst_static_pad_template_get(&gst_effectv_src_template), "src");
   gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
 
   filter->ripple = NULL;
