@@ -21,6 +21,8 @@
 #include "config.h"
 #endif
 
+#include <gst/bytestream/bytestream.h>
+
 #ifdef HAVE_MAD
 #include <id3tag.h>
 #endif
@@ -29,48 +31,32 @@
 #include "gstid3types.h"
 
 /* elementfactory information */
-static GstElementDetails id3types_details = {
+static GstElementDetails id3types_details = GST_ELEMENT_DETAILS (
   "ID3v1/v2 tag parser",
   "Codec/Parser",
-#ifdef HAVE_MAD
-  "GPL", /* grmbl, where's a good C LGPL ID3v1/2 reader? */
-#else
-  "LGPL",
-#endif
-  "Parses ID3v1 and ID3v2 tags",
-  VERSION,
-  "Erik Walthinsen <omega@cse.ogi.edu>\n"
-  "Ronald Bultje <rbultje@ronald.bitfreak.net>",
-  "(C) 1999-2003",
-};
+  "parse metadata tags often found in mp3s",
+  "Erik Walthinsen <omega@cse.ogi.edu>, "
+  "Ronald Bultje <rbultje@ronald.bitfreak.net>"
+);
 
-GST_PAD_TEMPLATE_FACTORY (sink_templ,
+static GstStaticPadTemplate sink_templ =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "id3types_sink",
-    "audio/x-id3",
-      "id3version", GST_PROPS_INT_RANGE (1, 2)
+  GST_STATIC_CAPS ( "application/x-id3, "
+      "id3version = (int) [ 1, 2]"
   )
 );
 
-GST_PAD_TEMPLATE_FACTORY (src_templ,
+static GstStaticPadTemplate src_templ =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_SOMETIMES,
-  NULL
+  GST_STATIC_CAPS_ANY
 );
 
-static GstCaps* id3_type_find (GstByteStream *bs, gpointer private);
-
-static GstTypeDefinition id3type_definition = {
-  "id3_audio/x-id3",
-  "audio/x-id3",
-  ".mp3 .mp2 .mp1 .mpga .ogg .flac", /* probably more... *sigh* */
-  id3_type_find,
-};
-
 /* signals */
 enum {
   /* FILL ME */
@@ -85,12 +71,13 @@ enum {
 };
 
 
+static void	gst_id3types_base_init		(gpointer g_class);
 static void	gst_id3types_class_init		(GstID3TypesClass *klass);
 static void	gst_id3types_init		(GstID3Types  *mp3parse);
 
 static const GstEventMask *
 		gst_id3types_event_mask		(GstPad       *pad);
-static gboolean	gst_id3types_event		(GstPad       *pad,
+static gboolean	gst_id3types_src_event		(GstPad       *pad,
 						 GstEvent     *event);
 
 static const GstQueryType *
@@ -122,7 +109,7 @@ gst_id3types_get_type (void)
   if (!id3types_type) {
     static const GTypeInfo id3types_info = {
       sizeof (GstID3TypesClass),
-      NULL,
+      gst_id3types_base_init,
       NULL,
       (GClassInitFunc) gst_id3types_class_init,
       NULL,
@@ -192,30 +179,15 @@ gst_id3types_detect (GstByteStream *bs,
   return res;
 }
 
-static GstCaps *
-id3_type_find (GstByteStream *bs,
-	       gpointer       private) 
+static void
+gst_id3types_base_init (gpointer g_class)
 {
-  GstCaps *new = NULL;
-  guint version, size;
-
-  if (gst_id3types_detect (bs, &version, &size)) {
-    switch (version) {
-      case 1: case 2:
-        /* ID3v1/v2 tags */
-        GST_DEBUG ("id3typefind: detected ID3 Tag V%d", version);
-        new = GST_CAPS_NEW ("id3_type_find",
-			    "audio/x-id3",
-			      "id3version", GST_PROPS_INT (version));
-        break;
-      default:
-        break;
-    }
-  }
-
-  return new;
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&sink_templ));
+  gst_element_class_add_pad_template (element_class, gst_static_pad_template_get (&src_templ));
+  gst_element_class_set_details (element_class, &id3types_details);
 }
-
 static void
 gst_id3types_class_init (GstID3TypesClass *klass)
 {
@@ -227,7 +199,7 @@ gst_id3types_class_init (GstID3TypesClass *klass)
 
   g_object_class_install_property (gobject_class, ARG_METADATA,
     g_param_spec_boxed ("metadata", "Metadata", "Metadata",
-                        GST_TYPE_CAPS, G_PARAM_READABLE));
+                        GST_TYPE_STRUCTURE, G_PARAM_READABLE));
 
   parent_class = g_type_class_ref (GST_TYPE_ELEMENT);
 
@@ -253,7 +225,8 @@ gst_id3types_cont (GstID3Types *id3)
 static inline void
 gst_id3types_stop (GstID3Types *id3)
 {
-  gst_caps_replace (&id3->metadata, NULL);
+  if (id3->metadata) gst_structure_free (id3->metadata);
+  id3->metadata = NULL;
   id3->pass_through = TRUE;
 }
 
@@ -262,49 +235,32 @@ gst_id3types_init (GstID3Types *id3)
 {
   GST_FLAG_SET (id3, GST_ELEMENT_EVENT_AWARE);
 
-  id3->sinkpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (sink_templ),
-					    "sink");
+  id3->sinkpad = gst_pad_new_from_template (
+      gst_static_pad_template_get (&sink_templ), "sink");
   gst_element_set_loop_function (GST_ELEMENT (id3), gst_id3types_loop);
-  gst_pad_set_event_function (id3->sinkpad, gst_id3types_event);
-  gst_pad_set_event_mask_function (id3->sinkpad, gst_id3types_event_mask);
   gst_id3types_start (id3);
   gst_element_add_pad (GST_ELEMENT (id3), id3->sinkpad);
 
   id3->metadata = NULL;
   id3->id3_tag_size = 0;
+
+  GST_FLAG_SET (id3, GST_ELEMENT_EVENT_AWARE);
 }
 
 static const GstEventMask *
 gst_id3types_event_mask (GstPad *pad)
 {
-  GstID3Types *id3 = GST_ID3TYPES (gst_pad_get_parent (pad));
-  const GstEventMask *masks = NULL;
+  static const GstEventMask my_masks[] = {
+    { GST_EVENT_SEEK, GST_SEEK_METHOD_SET },
+    { 0, }
+  };
 
-  if (pad == id3->sinkpad) {
-    static const GstEventMask my_masks[] = {
-      { GST_EVENT_DISCONTINUOUS, 0 },
-      { 0, }
-    };
-    masks = my_masks;
-  } else if (pad == id3->srcpad) {
-    static const GstEventMask my_masks[] = {
-      { GST_EVENT_SEEK, GST_SEEK_METHOD_SET },
-      { 0, }
-    };
-    masks = my_masks;
-  } else {
-    g_assert (0);
-  }
-
-  return masks;
+  return my_masks;
 }
-
-static gboolean
-gst_id3types_event (GstPad   *pad,
-		    GstEvent *event)
+static void
+gst_id3types_sink_event (GstPad *pad, GstEvent *event)
 {
   GstID3Types *id3 = GST_ID3TYPES (gst_pad_get_parent (pad));
-  gboolean res = TRUE;
 
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_DISCONTINUOUS: {
@@ -336,6 +292,18 @@ gst_id3types_event (GstPad   *pad,
       gst_pad_event_default (pad, new);
       break;
     }
+    default:
+      gst_pad_event_default (pad, event);
+      break;
+  }
+}
+static gboolean
+gst_id3types_src_event (GstPad *pad, GstEvent *event)
+{
+  GstID3Types *id3 = GST_ID3TYPES (gst_pad_get_parent (pad));
+  gboolean res = TRUE;
+
+  switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
       /* comes from the src pad... */
       switch (GST_EVENT_SEEK_FORMAT (event)) {
@@ -408,7 +376,7 @@ gst_id3types_chain (GstPad    *pad,
   GstID3Types *id3 = GST_ID3TYPES (gst_pad_get_parent (pad));
 
   if (GST_IS_EVENT (buf)) {
-    gst_id3types_event (id3->sinkpad, GST_EVENT (buf));
+    gst_id3types_sink_event (id3->sinkpad, GST_EVENT (buf));
   } else {
     /* passthrough mode */
     gst_pad_push (id3->srcpad, GST_DATA (buf));
@@ -417,18 +385,14 @@ gst_id3types_chain (GstPad    *pad,
 
 #ifdef HAVE_MAD
 /* gracefuly ripped from madplay */
-static GstCaps *
+static GstStructure *
 id3_to_caps (struct id3_tag const *tag)
 {
   unsigned int i;
   struct id3_frame const *frame;
   id3_ucs4_t const *ucs4;
   id3_utf8_t *utf8;
-  GstProps *props;
-  GstPropsEntry *entry;
-  GstCaps *caps;
-  GList *values;
-
+  GstStructure *structure;
   struct {
     char const *id;
     char const *name;
@@ -450,10 +414,9 @@ id3_to_caps (struct id3_tag const *tag)
     { "TRSN",            "Station"      },
     { "TENC",            "Encoder"      },
   };
+  GValue value;
 
-  /* text information */
-  props = gst_props_empty_new ();
-
+  structure = gst_structure_new ("application/x-gst-metadata", NULL);
   for (i = 0; i < sizeof(info) / sizeof(info[0]); ++i) {
     union id3_field const *field;
     unsigned int nstrings, namelen, j;
@@ -469,10 +432,16 @@ id3_to_caps (struct id3_tag const *tag)
     name = info[i].name;
 
     if (name) {
+      memset (&value, 0, sizeof(value));
+
+      g_value_init (&value, GST_TYPE_LIST);
+
       namelen = name ? strlen(name) : 0;
 
-      values = NULL;
       for (j = 0; j < nstrings; ++j) {
+        GValue list_value = { 0 };
+	g_value_init (&list_value, G_TYPE_STRING);
+
         ucs4 = id3_field_getstrings(field, j);
         g_assert(ucs4);
 
@@ -483,28 +452,25 @@ id3_to_caps (struct id3_tag const *tag)
         if (utf8 == 0)
 	  goto fail;
 
-        entry = gst_props_entry_new (name, GST_PROPS_STRING_TYPE, utf8);
-	values = g_list_prepend (values, entry);
-        free(utf8);
+	g_value_set_string (&list_value, utf8);
+	gst_value_list_append_value (&value, &list_value);
       }
-      if (values) {
-        values = g_list_reverse (values);
-
-        if (g_list_length (values) == 1) {
-          gst_props_add_entry (props, (GstPropsEntry *) values->data);
-        }
-        else {
-          entry = gst_props_entry_new(name, GST_PROPS_GLIST_TYPE, values);
-          gst_props_add_entry (props, (GstPropsEntry *) entry);
-        }
-        g_list_free (values);
+      if (gst_value_list_get_size (&value) == 1) {
+	gst_structure_set_value (structure, name,
+	    gst_value_list_get_value (&value, 0));
+      } else if (gst_value_list_get_size (&value) > 1) {
+	gst_structure_set_value (structure, name, &value);
       }
     }
   }
 
-  values = NULL;
+  memset (&value, 0, sizeof(value));
+  g_value_init (&value, GST_TYPE_LIST);
   i = 0;
   while ((frame = id3_tag_findframe(tag, ID3_FRAME_COMMENT, i++))) {
+    GValue list_value = { 0 };
+    g_value_init (&list_value, G_TYPE_STRING);
+
     ucs4 = id3_field_getstring(&frame->fields[2]);
     g_assert(ucs4);
 
@@ -518,45 +484,79 @@ id3_to_caps (struct id3_tag const *tag)
     if (utf8 == 0)
       goto fail;
 
-    entry = gst_props_entry_new ("Comment", GST_PROPS_STRING_TYPE, utf8);
-    values = g_list_prepend (values, entry);
+    g_value_set_string (&list_value, utf8);
+    gst_value_list_append_value (&value, &list_value);
     free(utf8);
   }
-  if (values) {
-    values = g_list_reverse (values);
-
-    if (g_list_length (values) == 1) {
-      gst_props_add_entry (props, (GstPropsEntry *) values->data);
-    }
-    else {
-      entry = gst_props_entry_new("Comment", GST_PROPS_GLIST_TYPE, values);
-      gst_props_add_entry (props, (GstPropsEntry *) entry);
-    }
-    g_list_free (values);
+  if (gst_value_list_get_size (&value) == 1) {
+    gst_structure_set_value (structure, "Comment",
+	gst_value_list_get_value (&value, 0));
+  } else if (gst_value_list_get_size (&value) > 1) {
+    gst_structure_set_value (structure, "Comment", &value);
   }
 
-  gst_props_debug (props);
+  return structure;
 
-  caps = gst_caps_new ("mad_metadata",
-		       "application/x-gst-metadata",
-		       props);
-  if (0) {
 fail:
-    g_warning ("mad: could not parse ID3 tag");
+  g_warning ("mad: could not parse ID3 tag");
 
-    return NULL;
-  }
-
-  return caps;
+  return NULL;
 }
 #endif
 
+typedef struct {
+  GstByteStream *bs;
+  guint best_probability;
+  GstCaps *caps;
+  GstBuffer *buffer;
+} SimpleTypeFind;
+guint8 *
+simple_find_peek (gpointer data, gint64 offset, guint size)
+{
+  guint result;
+  SimpleTypeFind *find = (SimpleTypeFind *) data;
+  
+  if (offset < 0)
+    return NULL;
+  
+  if (find->buffer) {
+    if (GST_BUFFER_SIZE (find->buffer) >= offset + size) {
+      return GST_BUFFER_DATA (find->buffer) + offset;
+    } else {
+      gst_data_unref (GST_DATA (find->buffer));
+    }
+  }
+  if (offset >= 0) {
+    while ((result = gst_bytestream_peek (find->bs, &find->buffer, offset + size)) == 0) {
+      GstEvent *event = NULL;
+      gst_bytestream_get_status (find->bs, NULL, &event);
+      g_assert (event);
+      gst_id3types_sink_event (find->bs->pad, event);
+    }
+    if (result == offset + size) {
+      return GST_BUFFER_DATA (find->buffer) + offset;
+    }
+  }
+  return NULL;
+}
+static void
+simple_find_suggest (gpointer data, guint probability, const GstCaps *caps)
+{
+  SimpleTypeFind *find = (SimpleTypeFind *) data;
+
+  if (probability > find->best_probability) {
+    gst_caps_replace (&find->caps, gst_caps_copy(caps));
+    find->best_probability = probability;
+  }
+}
 static void
 gst_id3types_loop (GstElement *element)
 {
-  const GList *type_list = gst_type_get_list ();
+  GList *type_list, *walk;
   GstID3Types *id3;
   GstByteStream *bs;
+  GstTypeFind gst_find;
+  SimpleTypeFind find;
   GstBuffer *buf = NULL;
   guint size = 0, version = 0;
 #ifdef HAVE_MAD
@@ -574,32 +574,28 @@ gst_id3types_loop (GstElement *element)
   bs = gst_bytestream_new (id3->sinkpad);
 
   /* first loop... */
-  if (!gst_id3types_detect (bs, &version, &size)) {
-    gst_element_error (element,
-		       "No ID3 tag detected");
-    goto error;
-  }
-
-  /* OK: so we've now got a ID3 tag of that size, let's read it */
-  if (gst_bytestream_peek (bs, &buf, size) != size) {
-    gst_element_error (element,
-		       "Failed to read ID3 tag - hard disk on fire?");
-    goto error;
-  }
-  gst_bytestream_flush (bs, size);
-
+  if (gst_id3types_detect (bs, &version, &size)) {
+    /* OK: so we've now got a ID3 tag of that size, let's read it */
+    if (gst_bytestream_peek (bs, &buf, size) != size) {
+      GST_ELEMENT_ERROR (element, RESOURCE, READ, (NULL),
+			 ("Failed to read ID3 tag using bytestream"));
+      goto error;
+    }
+    gst_bytestream_flush (bs, size);
 #ifdef HAVE_MAD
-  /* here, we should parse the ID3 header and notify the app */
-  if (!(tag = id3_tag_parse (GST_BUFFER_DATA (buf), size))) {
-    gst_element_error (element,
-		       "Failed to parse ID3v1/2 tags");
-    goto error;
-  } else {
-    gst_caps_replace_sink (&id3->metadata, id3_to_caps (tag));
-    id3_tag_delete (tag);
-    g_object_notify (G_OBJECT (id3), "metadata");
-  }
+    /* here, we should parse the ID3 header and notify the app */
+    if (!(tag = id3_tag_parse (GST_BUFFER_DATA (buf), size))) {
+      GST_ELEMENT_ERROR (element, STREAM, DECODE, (NULL),
+			 ("Failed to parse ID3v1/2 tags"));
+      goto error;
+    } else {
+      if (id3->metadata) gst_structure_free (id3->metadata);
+      id3->metadata = id3_to_caps (tag);
+      id3_tag_delete (tag);
+      g_object_notify (G_OBJECT (id3), "metadata");
+    }
 #endif
+  }
 
   /* done */
   id3->id3_tag_size = size;
@@ -608,51 +604,49 @@ gst_id3types_loop (GstElement *element)
    * this id3 thingy... Please note that this is a cruel hack
    * for as long as spider doesn't support multi-type-finding.
    */
-  while (type_list) {
-    GSList *factories;
-    GstType *type = (GstType *) type_list->data;
+  walk = type_list = gst_type_find_factory_get_list ();
+  
+  find.bs = bs;
+  find.best_probability = 0;
+  find.caps = NULL;
+  find.buffer = NULL;
+  gst_find.data = &find;
+  gst_find.peek = simple_find_peek;
+  gst_find.suggest = simple_find_suggest;
+  while (walk) {
+    GstTypeFindFactory *factory = GST_TYPE_FIND_FACTORY (walk->data);
+    
+    gst_type_find_factory_call_function (factory, &gst_find);
+    if (find.best_probability >= GST_TYPE_FIND_MAXIMUM)
+      break;
+    walk = g_list_next (walk);
+  }
+  g_list_free (type_list);
+  if (find.best_probability > 0) {
+    id3->srcpad = gst_pad_new_from_template (gst_static_pad_template_get (&src_templ), "src");
+    gst_pad_set_event_function (id3->srcpad, gst_id3types_src_event);
+    gst_pad_set_event_mask_function (id3->srcpad, gst_id3types_event_mask);
+    gst_pad_set_query_type_function (id3->srcpad, gst_id3types_get_query_types);
+    gst_pad_set_query_function (id3->srcpad, gst_id3types_handle_query);
+    gst_pad_use_explicit_caps (id3->srcpad);
+    gst_pad_set_explicit_caps (id3->srcpad, find.caps);
+    gst_element_add_pad (GST_ELEMENT (id3), id3->srcpad);
 
-    factories = type->factories;
-
-    while (factories) {
-      GstTypeFactory *factory = GST_TYPE_FACTORY (factories->data);
-      GstTypeFindFunc typefindfunc = (GstTypeFindFunc) factory->typefindfunc;
-      GstCaps *caps;
-
-      if (typefindfunc && (caps = typefindfunc (bs, factory))) {
-        GST_DEBUG ("found type: %d \"%s\" \"%s\"\n",
-		   caps->id, type->mime, gst_caps_get_name (caps));
-
-        id3->srcpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (src_templ),
-						 "src");
-        gst_pad_set_event_function (id3->srcpad, gst_id3types_event);
-        gst_pad_set_event_mask_function (id3->srcpad, gst_id3types_event_mask);
-        gst_pad_set_query_type_function (id3->srcpad, gst_id3types_get_query_types);
-        gst_pad_set_query_function (id3->srcpad, gst_id3types_handle_query);
-        gst_pad_try_set_caps (id3->srcpad, caps);
-        gst_element_add_pad (GST_ELEMENT (id3), id3->srcpad);
-        goto tf_done;
-      }
-      factories = g_slist_next (factories);
+    if (bs->listavail) {
+      gst_bytestream_read (bs, &buf, bs->listavail);
+      gst_pad_push (id3->srcpad, GST_DATA (buf));
     }
-    type_list = g_list_next (type_list);
+    gst_id3types_cont (id3);
+  } else {
+    GST_ELEMENT_ERROR (element, STREAM, TYPE_NOT_FOUND, (NULL),
+		       ("Media stream type not detected after id3v1/2 header"));
   }
-  gst_element_error (element,
-		     "Media stream type not detected after id3v1/2 header");
-  goto error;
+  if (find.caps)
+    gst_caps_free (find.caps);
+  if (find.buffer)
+    gst_data_unref (GST_DATA (find.buffer));
 
-tf_done:
-  if (bs->listavail) {
-    gst_bytestream_read (bs, &buf, bs->listavail);
-    gst_pad_push (id3->srcpad, GST_DATA (buf));
-  }
-  gst_id3types_cont (id3);
 error:
-#ifdef HAVE_MAD
-  if (tag) {
-    id3_tag_delete (tag);
-  }
-#endif
   gst_bytestream_destroy (bs);
 }
 
@@ -704,35 +698,39 @@ gst_id3types_change_state (GstElement *element)
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstTypeFactory *type;
-  GstElementFactory *factory;
-
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
+      
   /* create an elementfactory for the id3types element */
-  factory = gst_element_factory_new ("id3types",
-		                     GST_TYPE_ID3TYPES,
-                                     &id3types_details);
-  g_return_val_if_fail (factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
-
-  gst_element_factory_add_pad_template (factory,
-	GST_PAD_TEMPLATE_GET (sink_templ));
-  gst_element_factory_add_pad_template (factory,
-	GST_PAD_TEMPLATE_GET (src_templ));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  /* and typefinding */
-  type = gst_type_factory_new (&id3type_definition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
-
+  if (!gst_element_register (plugin, "id3types", GST_RANK_SECONDARY, GST_TYPE_ID3TYPES))
+    return FALSE;
   return TRUE;
 }
 
-GstPluginDesc plugin_desc = {
+#ifdef HAVE_MAD
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "id3types",
-  plugin_init
-};
+  "Parses ID3v1 and ID3v2 tags",
+  plugin_init,
+  VERSION,
+  "GPL", /* grmbl, where's a good C LGPL ID3v1/2 reader? */
+  GST_PACKAGE,
+  GST_ORIGIN
+)
+#else
+GST_PLUGIN_DEFINE (
+  GST_VERSION_MAJOR,
+  GST_VERSION_MINOR,
+  "id3types",
+  "Parses ID3v1 and ID3v2 tags",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN
+)
+#endif
