@@ -19,9 +19,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif
 
 #include "gstfloat2int.h"
+#include "float22int.h"
 #include "gstint2float.h"
 
 static gboolean
@@ -30,7 +33,9 @@ plugin_init (GstPlugin *plugin)
   if (!gst_element_register (plugin, "int2float",
 			     GST_RANK_SECONDARY, GST_TYPE_INT2FLOAT) ||
       !gst_element_register (plugin, "float2int",
-			     GST_RANK_SECONDARY, GST_TYPE_FLOAT2INT))
+			     GST_RANK_SECONDARY, GST_TYPE_FLOAT2INT) ||
+      !gst_element_register (plugin, "float2intnew",
+			     GST_RANK_SECONDARY, GST_TYPE_FLOAT2_2_INT))
     return FALSE;
 
   return TRUE;
@@ -44,7 +49,6 @@ GST_PLUGIN_DEFINE (
   plugin_init,
   VERSION,
   GST_LICENSE,
-  GST_COPYRIGHT,
   GST_PACKAGE,
   GST_ORIGIN
 )
