@@ -22,7 +22,9 @@
  * Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
 #include <config.h>
+#endif
 #include <gst/gst.h>
 
 #define GST_TYPE_ONETON (gst_oneton_get_type ())
@@ -71,47 +73,38 @@ enum {
 	ARG_0,
 };
 
-static GstPadTemplate *
-oneton_sink_factory (void)
-{
-	return gst_pad_template_new (
-		"sink",
-		GST_PAD_SINK,
-		GST_PAD_ALWAYS,
-		gst_caps_new (
-			"int_n_channel_sink",
-			"audio/x-raw-int",
-			gst_props_new (
-				"endianness", GST_PROPS_INT (G_BYTE_ORDER),
-				"signed", GST_PROPS_BOOLEAN (TRUE),
-				"width", GST_PROPS_INT (16),
-				"depth", GST_PROPS_INT (16),
-				"rate", GST_PROPS_INT_RANGE (4000, 96000),
-				"channels", GST_PROPS_INT_RANGE (1, 4),
-				NULL)),
-		NULL);
-}
 
-static GstPadTemplate *
-oneton_src_factory (void)
-{
-	return gst_pad_template_new (
-		"src_%d",
-		GST_PAD_SRC,
-		GST_PAD_SOMETIMES,
-		gst_caps_new (
-			"int_n_channel_src",
-			"audio/x-raw-int",
-			gst_props_new (
-				"endianness", GST_PROPS_INT (G_BYTE_ORDER),
-				"signed", GST_PROPS_BOOLEAN (TRUE),
-				"width", GST_PROPS_INT (16),
-				"depth", GST_PROPS_INT (16),
-				"rate", GST_PROPS_INT_RANGE (4000, 96000),
-				"channels", GST_PROPS_INT (1),
-				NULL)),
-		NULL);
-}
+static GstStaticPadTemplate oneton_sink_template =
+GST_STATIC_PAD_TEMPLATE (
+    "sink",
+    GST_PAD_SINK,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (
+      "audio/x-raw-int, "
+      "endianness = (int) BYTE_ORDER, "
+      "signed = (boolean) TRUE, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = (int) [ 4000, 96000 ], "
+      "channels = (int) [ 1, 4 ]"
+    )
+);
+
+static GstStaticPadTemplate oneton_src_template =
+GST_STATIC_PAD_TEMPLATE (
+    "src",
+    GST_PAD_SRC,
+    GST_PAD_ALWAYS,
+    GST_STATIC_CAPS (
+      "audio/x-raw-int, "
+      "endianness = (int) BYTE_ORDER, "
+      "signed = (boolean) TRUE, "
+      "width = (int) 16, "
+      "depth = (int) 16, "
+      "rate = (int) [ 4000, 96000 ], "
+      "channels = (int) [ 1, 4 ]"
+    )
+);
 
 GType gst_oneton_get_type (void);
 
@@ -131,67 +124,74 @@ static void inline gst_oneton_fast_8bit_chain (gint8 *data,
 
 
 
-static GstPadTemplate *sinktemplate, *srctemplate;
 static GstElementClass *parent_class = NULL;
 
 static GstPadLinkReturn
-gst_oneton_connect (GstPad *pad,
-		    GstCaps *caps)
+gst_oneton_sink_connect (GstPad *pad, const GstCaps *caps)
 {
 	GstOneToN *oneton;
+	int i, new_chans, need;
+	GstCaps *srccaps;
+	GList *p;
+	GstStructure *structure;
 
 	oneton = GST_ONETON (gst_pad_get_parent (pad));
 
 	DEBUG;
 	g_return_val_if_fail (GST_IS_ONETON (oneton), GST_PAD_LINK_REFUSED);
 
-	if (!GST_CAPS_IS_FIXED (caps)) {
+	structure = gst_caps_get_structure (caps, 0);
+
+	/* Get the number of channels coming in */
+	gst_structure_get_int  (structure, "channels", &new_chans);
+	if (new_chans == 0) {
 		return GST_PAD_LINK_DELAYED;
 	}
 
-	if (pad == oneton->sinkpad) {
-		int i;
-		GstCaps *srccaps;
-		GList *p;
+	if (new_chans == oneton->channels) {
+		return GST_PAD_LINK_OK;
+	}
 
-		/* Get the number of channels coming in */
-		gst_caps_get_int (caps, "channels", &oneton->channels);
-		if (oneton->channels == 0) {
-			return GST_PAD_LINK_DELAYED;
-		}
+	need = new_chans - oneton->channels;
 
-		srccaps = gst_caps_copy (caps);
+	srccaps = gst_caps_copy (caps);
 
-		/* make sure we onw this copy */
-		gst_caps_ref (srccaps);
-		gst_caps_sink (srccaps);
+	gst_caps_set_simple (srccaps, "channels", G_TYPE_INT, 1, NULL);
+	gst_structure_get_int  (structure, "width", &oneton->width);
 
-		gst_caps_set (srccaps, "channels", GST_PROPS_INT (1));
-		gst_caps_get_int (caps, "width", &oneton->width);
-
-		/* Disconnect any pads that already exist */
-		for (p = oneton->srcpads; p; p = p->next) {
+	if (need < 0) {
+		for (p = g_list_last (oneton->srcpads); p;) {
 			GstPad *peer, *pad = p->data;
-
-			peer = GST_PAD_PEER (pad);
+			GList *old;
 			
+			peer = GST_PAD_PEER (pad);
+	
 			/* Check if they're connected */
 			if (peer) {
 				gst_pad_unlink (pad, peer);
 			}
-
+	
 			gst_element_remove_pad (GST_ELEMENT (oneton), pad);
+
+			old = p;
+			p = p->prev;
+			
+			g_list_free_1 (old);
+			if (p != NULL) {
+				/* Remove link to the next */
+				p->next = NULL;
+			}
 		}
-		g_list_free (oneton->srcpads);
-		oneton->srcpads = NULL;
-		
+	} else {	
 		/* Create that number of src pads */
-		for (i = 0; i < oneton->channels; i++) {
+		for (i = oneton->channels; i < new_chans; i++) {
 			GstPad *pad;
 			char *pad_name;
 
 			pad_name = g_strdup_printf ("src_%d", i);
-			pad = gst_pad_new_from_template (srctemplate, pad_name);
+			pad = gst_pad_new_from_template (
+        	            gst_static_pad_template_get (&oneton_src_template),
+     					                 pad_name);
 			g_free (pad_name);
 
 			/* FIXME handle DELAYED too */
@@ -202,11 +202,11 @@ gst_oneton_connect (GstPad *pad,
 
 			oneton->srcpads = g_list_append (oneton->srcpads, pad);
 		}
-
-		/* we don't need the copy anymore, caps will be owned by the pads
-		 * when negotiation was successful */
-		gst_caps_unref (srccaps);
 	}
+	
+	oneton->channels = new_chans;
+	
+	gst_caps_free (srccaps);
 
 	return GST_PAD_LINK_OK;
 }
@@ -235,8 +235,10 @@ gst_oneton_base_init (GstOneToNClass *klass)
 {
 	GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-	gst_element_class_add_pad_template (element_class, srctemplate);
-	gst_element_class_add_pad_template (element_class, sinktemplate);
+	gst_element_class_add_pad_template (element_class,
+            gst_static_pad_template_get (&oneton_src_template));
+	gst_element_class_add_pad_template (element_class,
+            gst_static_pad_template_get (&oneton_sink_template));
 	gst_element_class_set_details (element_class, &oneton_details);
 }
 
@@ -256,9 +258,10 @@ static void
 gst_oneton_init (GstOneToN *oneton)
 {
 	DEBUG;
-	oneton->sinkpad = gst_pad_new_from_template (sinktemplate, "sink");
+	oneton->sinkpad = gst_pad_new_from_template (
+            gst_static_pad_template_get (&oneton_sink_template), "sink");
 	gst_pad_set_chain_function (oneton->sinkpad, gst_oneton_chain);
-	gst_pad_set_link_function (oneton->sinkpad, gst_oneton_connect);
+	gst_pad_set_link_function (oneton->sinkpad, gst_oneton_sink_connect);
 	gst_element_add_pad (GST_ELEMENT (oneton), oneton->sinkpad);
 
 	oneton->channels = 0;
@@ -331,8 +334,8 @@ gst_oneton_chain (GstPad *pad,
 		break;
 
 	default:
-		gst_element_error (GST_ELEMENT (oneton),
-				   "oneton: capsnego was never performed, bailing...");
+		GST_ELEMENT_ERROR (oneton, CORE, NEGOTIATION, (NULL),
+                      ("format wasn't negotiated before chain function"));
 		return;
 	}
 
@@ -385,9 +388,6 @@ plugin_init (GstPlugin *plugin)
 {
 	DEBUG;
 
-	srctemplate = oneton_src_factory ();
-	sinktemplate = oneton_sink_factory ();
-
 	return gst_element_register (plugin, "oneton",
 				     GST_RANK_NONE,
 				     GST_TYPE_ONETON);
