@@ -78,22 +78,10 @@ enum GstRMDemuxStreamType {
 static GstElementDetails 
 gst_rmdemux_details = 
 {
-  "QuickTime Demuxer",
+  "RealMedia Demuxer",
   "Codec/Demuxer",
-  "LGPL",
   "Demultiplex a RealMedia file into audio and video streams",
-  VERSION,
-  "David Schleef <ds@schleef.org>",
-  "(C) 2003",
-};
-
-static GstCaps* realmedia_type_find (GstByteStream *bs, gpointer private);
-
-static GstTypeDefinition realmediadefinition = {
-  "rmdemux_video/realmedia",
-  "video/x-pn-realvideo", 
-  ".ram .rm .ra",
-  realmedia_type_find,
+  "David Schleef <ds@schleef.org>"
 };
 
 enum {
@@ -104,34 +92,34 @@ enum {
   ARG_0
 };
 
-GST_PAD_TEMPLATE_FACTORY (sink_templ,
+static GstStaticPadTemplate gst_rmdemux_sink_template =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "rmdemux_sink",
-     "video/x-pn-realvideo", 
-     NULL
-  )
-)
+  GST_STATIC_CAPS ("video/x-pn-realvideo")
+);
 
-GST_PAD_TEMPLATE_FACTORY (src_video_templ,
+static GstStaticPadTemplate gst_rmdemux_videosrc_template =
+GST_STATIC_PAD_TEMPLATE (
   "video_%02d",
   GST_PAD_SRC,
   GST_PAD_SOMETIMES,
-  NULL
-)
+  GST_STATIC_CAPS_ANY
+);
 
-GST_PAD_TEMPLATE_FACTORY (src_audio_templ,
+static GstStaticPadTemplate gst_rmdemux_audiosrc_template =
+GST_STATIC_PAD_TEMPLATE (
   "audio_%02d",
   GST_PAD_SRC,
   GST_PAD_SOMETIMES,
-  NULL
-)
+  GST_STATIC_CAPS_ANY
+);
 
 static GstElementClass *parent_class = NULL;
 
 static void gst_rmdemux_class_init (GstRMDemuxClass *klass);
+static void gst_rmdemux_base_init (GstRMDemuxClass *klass);
 static void gst_rmdemux_init (GstRMDemux *rmdemux);
 static GstElementStateReturn gst_rmdemux_change_state(GstElement *element);
 static void gst_rmdemux_loop (GstElement *element);
@@ -163,7 +151,8 @@ static GType gst_rmdemux_get_type (void)
 
   if (!rmdemux_type) {
     static const GTypeInfo rmdemux_info = {
-      sizeof(GstRMDemuxClass), NULL, NULL,
+      sizeof(GstRMDemuxClass),
+      (GBaseInitFunc)gst_rmdemux_base_init, NULL,
       (GClassInitFunc)gst_rmdemux_class_init,
       NULL, NULL, sizeof(GstRMDemux), 0,
       (GInstanceInitFunc)gst_rmdemux_init,
@@ -173,6 +162,19 @@ static GType gst_rmdemux_get_type (void)
   return rmdemux_type;
 }
 
+static void gst_rmdemux_base_init (GstRMDemuxClass *klass)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
+
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rmdemux_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rmdemux_videosrc_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rmdemux_audiosrc_template));
+  gst_element_class_set_details (element_class, &gst_rmdemux_details);
+}
+
 static void gst_rmdemux_class_init (GstRMDemuxClass *klass) 
 {
   GObjectClass *gobject_class;
@@ -189,63 +191,33 @@ static void gst_rmdemux_class_init (GstRMDemuxClass *klass)
 static void 
 gst_rmdemux_init (GstRMDemux *rmdemux) 
 {
-  rmdemux->sinkpad = gst_pad_new_from_template (GST_PAD_TEMPLATE_GET (sink_templ), "sink");
+  rmdemux->sinkpad = gst_pad_new_from_template (
+      gst_static_pad_template_get (&gst_rmdemux_sink_template), "sink");
   gst_element_set_loop_function (GST_ELEMENT (rmdemux), gst_rmdemux_loop);
   gst_element_add_pad (GST_ELEMENT (rmdemux), rmdemux->sinkpad);
 }
 
-static GstCaps*
-realmedia_type_find (GstByteStream *bs, gpointer private)
-{
-  GstBuffer *buf = NULL;
-  GstCaps *new = NULL;
-
-  if (gst_bytestream_peek (bs, &buf, 4) == 4) {
-    gchar *data = GST_BUFFER_DATA (buf);
-
-    if (!strncmp (data, ".RMF", 4)) {
-      new = GST_CAPS_NEW ("realmedia_type_find",
-			  "application/vnd.rn-realmedia", 
-			    NULL);
-    }
-  }
-
-  if (buf != NULL) {
-    gst_buffer_unref (buf);
-  }
-
-  return new;
-}
-
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-  GstTypeFactory *type;
+  if (!gst_library_load ("gstbytestream"))
+    return FALSE;
 
-  factory = gst_element_factory_new ("rmdemux", GST_TYPE_RMDEMUX,
-                                     &gst_rmdemux_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  gst_element_factory_set_rank (factory, GST_ELEMENT_RANK_PRIMARY);
-
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (sink_templ));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_video_templ));
-  gst_element_factory_add_pad_template (factory, GST_PAD_TEMPLATE_GET (src_audio_templ));
-
-  type = gst_type_factory_new (&realmediadefinition);
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (type));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "rmdemux",
+			       GST_RANK_PRIMARY, GST_TYPE_RMDEMUX);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "rmdemux",
-  plugin_init
-};
+  "Realmedia stream demuxer",
+  plugin_init,
+  VERSION,
+  "LGPL",
+  GST_PACKAGE,
+  GST_ORIGIN
+)
 
 static gboolean gst_rmdemux_handle_sink_event (GstRMDemux *rmdemux)
 {
@@ -490,60 +462,6 @@ static void gst_rmdemux_loop (GstElement *element)
 
 }
 
-static GstCaps *gst_rmdemux_src_getcaps(GstPad *pad, GstCaps *caps)
-{
-  GstRMDemux *rmdemux;
-  GstRMDemuxStream *stream;
-  int i;
-
-  GST_DEBUG ("gst_rmdemux_src_getcaps");
-
-  rmdemux = GST_RMDEMUX(gst_pad_get_parent(pad));
-
-  g_return_val_if_fail(GST_IS_RMDEMUX(rmdemux), NULL);
-
-  GST_DEBUG ("looking for pad %p in rmdemux %p", pad, rmdemux);
-  GST_DEBUG ("n_streams is %d", rmdemux->n_streams);
-  for(i=0;i<rmdemux->n_streams;i++){
-    stream = rmdemux->streams[i];
-    if(stream->pad == pad){
-      return stream->caps;
-    }
-  }
-
-  GST_DEBUG ("Couldn't find stream cooresponding to pad\n");
-
-  return NULL;
-}
-
-static GstPadLinkReturn
-gst_rmdemux_src_link(GstPad *pad, GstCaps *caps)
-{
-  GstRMDemux *rmdemux;
-  GstRMDemuxStream *stream;
-  int i;
-
-  GST_DEBUG ("gst_rmdemux_src_link");
-
-  rmdemux = GST_RMDEMUX(gst_pad_get_parent(pad));
-
-  GST_DEBUG ("looking for pad %p in rmdemux %p", pad, rmdemux);
-  g_return_val_if_fail(GST_IS_RMDEMUX(rmdemux), GST_PAD_LINK_REFUSED);
-
-  GST_DEBUG ("n_streams is %d\n", rmdemux->n_streams);
-  for(i=0;i<rmdemux->n_streams;i++){
-    stream = rmdemux->streams[i];
-    GST_DEBUG ("pad[%d] is %p\n", i, stream->pad);
-    if(stream->pad == pad){
-      return GST_PAD_LINK_OK;
-    }
-  }
-
-  GST_DEBUG ("Couldn't find stream cooresponding to pad\n");
-
-  return GST_PAD_LINK_REFUSED;
-}
-
 static GstRMDemuxStream *gst_rmdemux_get_stream_by_id(GstRMDemux *rmdemux,
     int id)
 {
@@ -564,51 +482,42 @@ void gst_rmdemux_add_stream(GstRMDemux *rmdemux, GstRMDemuxStream *stream)
 {
   if(stream->subtype == GST_RMDEMUX_STREAM_VIDEO){
     stream->pad = gst_pad_new_from_template (
-        GST_PAD_TEMPLATE_GET (src_video_templ), g_strdup_printf ("video_%02d",
-	  rmdemux->n_video_streams));
-    if(stream->caps){
-      gst_caps_set(stream->caps,"width",GST_PROPS_INT(stream->width));
-      gst_caps_set(stream->caps,"height",GST_PROPS_INT(stream->height));
+	gst_static_pad_template_get (&gst_rmdemux_videosrc_template),
+        g_strdup_printf ("video_%02d", rmdemux->n_video_streams));
+    if(stream->caps) {
+      gst_caps_set_simple (stream->caps,
+	  "width", G_TYPE_INT, stream->width,
+	  "height", G_TYPE_INT, stream->height, NULL);
     }
     rmdemux->n_video_streams++;
   }else
   if(stream->subtype == GST_RMDEMUX_STREAM_AUDIO){
     stream->pad = gst_pad_new_from_template (
-        GST_PAD_TEMPLATE_GET (src_audio_templ), g_strdup_printf ("audio_%02d",
-	  rmdemux->n_audio_streams));
-    stream->caps = GST_CAPS_NEW("audio_caps","audio/a52",NULL);
-    gst_caps_ref(stream->caps);
-    gst_caps_sink(stream->caps);
-    if(stream->caps){
-      if(gst_caps_has_property(stream->caps,"rate")){
-        gst_caps_set(stream->caps,"rate",GST_PROPS_INT((int)stream->rate));
-      }
-      if(gst_caps_has_property(stream->caps,"channels")){
-        gst_caps_set(stream->caps,"channels",GST_PROPS_INT(stream->n_channels));
-      }
-    }
+	gst_static_pad_template_get (&gst_rmdemux_audiosrc_template),
+        g_strdup_printf ("audio_%02d", rmdemux->n_audio_streams));
+    stream->caps = gst_caps_new_simple("audio/a52",NULL);
+    gst_caps_set_simple (stream->caps,
+	"rate", G_TYPE_INT, (int)stream->rate,
+	"channels", G_TYPE_INT, stream->n_channels, NULL);
     rmdemux->n_audio_streams++;
   }else{
     g_print("not adding stream of type %d\n",stream->subtype);
   }
 
+  GST_PAD_ELEMENT_PRIVATE (stream->pad) = stream;
   rmdemux->streams[rmdemux->n_streams] = stream;
   rmdemux->n_streams++;
   g_print("n_streams is now %d\n", rmdemux->n_streams);
 
   if(stream->pad){
-    gst_pad_set_getcaps_function(stream->pad, gst_rmdemux_src_getcaps);
-    gst_pad_set_link_function(stream->pad, gst_rmdemux_src_link);
+    gst_pad_use_explicit_caps (stream->pad);
 
-    g_print("adding pad %p to rmdemux %p\n", stream->pad, rmdemux);
+    GST_DEBUG ("setting caps: " GST_PTR_FORMAT, stream->caps);
+
+    gst_pad_set_explicit_caps(stream->pad, stream->caps);
+
+    GST_DEBUG ("adding pad %p to rmdemux %p", stream->pad, rmdemux);
     gst_element_add_pad(GST_ELEMENT (rmdemux), stream->pad);
-
-    /* Note: we need to have everything set up before calling try_set_caps */
-    if(stream->caps){
-      g_print("setting caps to %s\n",gst_caps_to_string(stream->caps));
-
-      gst_pad_try_set_caps(stream->pad, stream->caps);
-    }
   }
 }
 
