@@ -26,15 +26,12 @@
 #include <gst/video/video.h>
 #include "gsty4mencode.h"
 
-static GstElementDetails y4mencode_details = {
+static GstElementDetails y4mencode_details = GST_ELEMENT_DETAILS (
   "Y4mEncode",
-  "Codec/Video/Encoder",
-  "LGPL",
+  "Codec/Encoder/Video",
   "Encodes a YUV frame into the yuv4mpeg format (mjpegtools)",
-  VERSION,
-  "Wim Taymans <wim.taymans@chello.be>",
-  "(C) 2001",
-};
+  "Wim Taymans <wim.taymans@chello.be>"
+);
 
 
 /* Filter signals and args */
@@ -47,29 +44,25 @@ enum {
   ARG_0
 };
 
-GST_PAD_TEMPLATE_FACTORY (y4mencode_src_factory,
+static GstStaticPadTemplate y4mencode_src_factory =
+GST_STATIC_PAD_TEMPLATE (
   "src",
   GST_PAD_SRC,
   GST_PAD_ALWAYS,
-  GST_CAPS_NEW (
-    "test_src",
-    "application/x-yuv4mpeg",
-      "y4mversion", GST_PROPS_INT (1)
+  GST_STATIC_CAPS ("application/x-yuv4mpeg, "
+      "y4mversion = (int) 1"
   )
-)
+);
 
-GST_PAD_TEMPLATE_FACTORY (y4mencode_sink_factory,
+static GstStaticPadTemplate y4mencode_sink_factory =
+GST_STATIC_PAD_TEMPLATE (
   "sink",
   GST_PAD_SINK,
   GST_PAD_ALWAYS,
-  gst_caps_new (
-    "test_src",
-    "video/x-raw-yuv",
-      GST_VIDEO_YUV_PAD_TEMPLATE_PROPS (
-	      GST_PROPS_FOURCC (GST_STR_FOURCC ("I420")))
-  )
-)
+  GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("I420"))
+);
 
+static void	gst_y4mencode_base_init		(gpointer g_class);
 static void	gst_y4mencode_class_init	(GstY4mEncodeClass *klass);
 static void	gst_y4mencode_init		(GstY4mEncode *filter);
 
@@ -98,7 +91,7 @@ gst_y4mencode_get_type(void) {
   if (!y4mencode_type) {
     static const GTypeInfo y4mencode_info = {
       sizeof(GstY4mEncodeClass),
-      NULL,
+      gst_y4mencode_base_init,
       NULL,
       (GClassInitFunc)gst_y4mencode_class_init,
       NULL,
@@ -114,6 +107,18 @@ gst_y4mencode_get_type(void) {
   return y4mencode_type;
 }
 
+
+static void
+gst_y4mencode_base_init (gpointer g_class)
+{
+  GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
+  
+  gst_element_class_add_pad_template (element_class, 
+		  gst_static_pad_template_get (&y4mencode_src_factory));
+  gst_element_class_add_pad_template (element_class, 
+		  gst_static_pad_template_get (&y4mencode_sink_factory));
+  gst_element_class_set_details (element_class, &y4mencode_details);
+}
 static void
 gst_y4mencode_class_init (GstY4mEncodeClass *klass)
 {
@@ -132,12 +137,12 @@ gst_y4mencode_class_init (GstY4mEncodeClass *klass)
 }
 
 static GstPadLinkReturn
-gst_y4mencode_sinkconnect (GstPad *pad, GstCaps *caps)
+gst_y4mencode_sinkconnect (GstPad *pad, const GstCaps *caps)
 {
   GstY4mEncode *filter;
   gint idx = -1, i;
-  gfloat fps;
-  float framerates[] = {
+  gdouble fps;
+  gdouble framerates[] = {
     00.000,
     23.976, 24.000,              /* 24fps movie */
     25.000,                      /* PAL */
@@ -145,22 +150,22 @@ gst_y4mencode_sinkconnect (GstPad *pad, GstCaps *caps)
     50.000,
     59.940, 60.000
   };
+  GstStructure *structure;
 
   filter = GST_Y4MENCODE (gst_pad_get_parent (pad));
 
-  if (!GST_CAPS_IS_FIXED (caps))
-    return GST_PAD_LINK_DELAYED;
+  structure = gst_caps_get_structure (caps, 0);
 
-  gst_caps_get_int (caps, "width", &filter->width);
-  gst_caps_get_int (caps, "height", &filter->height);
-  gst_caps_get_float (caps, "framerate", &fps);
+  gst_structure_get_int  (structure, "width", &filter->width);
+  gst_structure_get_int  (structure, "height", &filter->height);
+  gst_structure_get_double  (structure, "framerate", &fps);
 
   /* find fps idx */
   for (i = 1; i < 9; i++) {
     if (idx == -1) {
        idx = i;
     } else {
-      gfloat old_diff = fabs(framerates[idx] - fps),
+      gdouble old_diff = fabs(framerates[idx] - fps),
              new_diff = fabs(framerates[i]   - fps);
 
       if (new_diff < old_diff) {
@@ -177,13 +182,13 @@ static void
 gst_y4mencode_init (GstY4mEncode *filter)
 {
   filter->sinkpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (y4mencode_sink_factory), "sink");
+		  gst_static_pad_template_get (&y4mencode_sink_factory), "sink");
   gst_element_add_pad (GST_ELEMENT (filter), filter->sinkpad);
   gst_pad_set_chain_function (filter->sinkpad, gst_y4mencode_chain);
   gst_pad_set_link_function (filter->sinkpad, gst_y4mencode_sinkconnect);
 
   filter->srcpad = gst_pad_new_from_template(
-		  GST_PAD_TEMPLATE_GET (y4mencode_src_factory), "src");
+		  gst_static_pad_template_get (&y4mencode_src_factory), "src");
   gst_element_add_pad (GST_ELEMENT (filter), filter->srcpad);
 
   filter->init = TRUE;
@@ -280,27 +285,19 @@ gst_y4mencode_change_state (GstElement *element)
 }
 
 static gboolean
-plugin_init (GModule *module, GstPlugin *plugin)
+plugin_init (GstPlugin *plugin)
 {
-  GstElementFactory *factory;
-
-  factory = gst_element_factory_new("y4menc",GST_TYPE_Y4MENCODE,
-                                   &y4mencode_details);
-  g_return_val_if_fail(factory != NULL, FALSE);
-  
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (y4mencode_src_factory));
-  gst_element_factory_add_pad_template (factory, 
-		  GST_PAD_TEMPLATE_GET (y4mencode_sink_factory));
-
-  gst_plugin_add_feature (plugin, GST_PLUGIN_FEATURE (factory));
-
-  return TRUE;
+  return gst_element_register (plugin, "y4menc", GST_RANK_NONE, GST_TYPE_Y4MENCODE);
 }
 
-GstPluginDesc plugin_desc = {
+GST_PLUGIN_DEFINE (
   GST_VERSION_MAJOR,
   GST_VERSION_MINOR,
   "y4menc",
-  plugin_init
-};
+  "Encodes a YUV frame into the yuv4mpeg format (mjpegtools)",
+  plugin_init,
+  VERSION,
+  GST_LICENSE,
+  GST_PACKAGE,
+  GST_ORIGIN
+)
